@@ -604,14 +604,13 @@ ebConnect(void)
     if(sql_database)
 	return true;
     if(!dbarea) {
-	setError
-	   ("the config file does not specify the database - cannot connect");
+	setError(MSG_DBUnspecified);
 	return false;
     }
     sql_exclist(exclist);
     sql_connect(dbarea, dblogin, dbpw);
     if(rv_lastStatus) {
-	setError("cannot connect to the database - error %d", rv_vendorStatus);
+	setError(MSG_DBConnect, rv_vendorStatus);
 	return false;
     }
     if(!sql_database)
@@ -632,13 +631,12 @@ static int scllen;
 static char *wcl;		/* where clause */
 static int wcllen;
 static char wherecol[COLNAMELEN + 2];
-static const char synwhere[] = "syntax error in where clause";
 static struct DBTABLE *td;
 
 static void
 unexpected(void)
 {
-    setError("unexpected sql error %d", rv_vendorStatus);
+    setError(MSG_DBUnexpected, rv_vendorStatus);
 }				/* unexpected */
 
 static void
@@ -671,14 +669,13 @@ buildWhereClause(void)
     e = strchr(w, '=');
     if(!e) {
 	if(!td->key1) {
-	    setError("no key column specified");
+	    setError(MSG_DBNoKey);
 	    return false;
 	}
 	e = td->cols[td->key1 - 1];
 	l = strlen(e);
 	if(l > COLNAMELEN) {
-	    setError("column name %s is too long, limit %d characters", e,
-	       COLNAMELEN);
+	    setError(MSG_DBColumnLong, e, COLNAMELEN);
 	    return false;
 	}
 	strcpy(wherecol, e);
@@ -686,11 +683,11 @@ buildWhereClause(void)
     } else if(isdigit(*w)) {
 	n = strtol(w, (char **)&w, 10);
 	if(w != e) {
-	    setError(synwhere);
+	    setError(MSG_DBSyntax);
 	    return false;
 	}
 	if(n == 0 || n > td->ncols) {
-	    setError("column %d is out of range", n);
+	    setError(MSG_DBColRange, n);
 	    return false;
 	}
 	goto setcol_n;
@@ -703,22 +700,21 @@ buildWhereClause(void)
 		if(!strstr(td->cols[i], wherecol))
 		    continue;
 		if(n) {
-		    setError("multiple columns match %s", wherecol);
+		    setError(MSG_DBManyColumns, wherecol);
 		    return false;
 		}
 		n = i + 1;
 	    }
 	}
 	if(!n) {
-	    setError("no column matches %s", wherecol);
+	    setError(MSG_DBNoColumn, wherecol);
 	    return false;
 	}
       setcol_n:
 	w = td->cols[n - 1];
 	l = strlen(w);
 	if(l > COLNAMELEN) {
-	    setError("column name %s is too long, limit %d characters", w,
-	       COLNAMELEN);
+	    setError(MSG_DBColumnLong, w, COLNAMELEN);
 	    return false;
 	}
 	strcpy(wherecol, w);
@@ -773,9 +769,9 @@ setTable(void)
 	    nzFree(scl);
 	    if(rv_lastStatus) {
 		if(rv_lastStatus == EXCNOTABLE)
-		    setError("no such table %s", td->name);
+		    setError(MSG_DBNoTable, td->name);
 		else if(rv_lastStatus == EXCNOCOLUMN)
-		    setError("invalid column name");
+		    setError(MSG_DBBadColumn);
 		else
 		    unexpected();
 		return false;
@@ -790,7 +786,7 @@ setTable(void)
 	cid = sql_prepare("select * from %s", myTab);
 	if(rv_lastStatus) {
 	    if(rv_lastStatus == EXCNOTABLE)
-		setError("no such table %s", myTab);
+		setError(MSG_DBNoTable, myTab);
 	    else
 		unexpected();
 	    return false;
@@ -826,7 +822,7 @@ setTable(void)
     if(s)
 	s = strpbrk(s + 1, "BT");
     if(s) {
-	setError("cannot select more than one blob column");
+	setError(MSG_DBManyBlobs);
 	return false;
     }
 
@@ -917,12 +913,11 @@ sqlReadRows(const char *filename, char **bufptr)
 	while(sql_fetchNext(cid, 0)) {
 	    unld = sql_mkunld('\177');
 	    if(strchr(unld, '|')) {
-		setError
-		   ("the data contains pipes, which is my reserved delimiter");
+		setError(MSG_DBPipes);
 		goto abort;
 	    }
 	    if(strchr(unld, '\n')) {
-		setError("the data contains newlines");
+		setError(MSG_DBNewline);
 		goto abort;
 	    }
 	    for(s = unld; *s; ++s)
@@ -989,32 +984,25 @@ intoFields(char *line)
 	    break;
 	if(j < td->ncols)
 	    continue;
-	setError
-	   ("line contains too many fields, please do not introduce any pipes into the text");
+	setError(MSG_DBAddField);
 	return false;
     }
 
     if(j == td->ncols)
 	return true;
-    setError
-       ("line contains too few fields, please do not remove any pipes from the text");
+    setError(MSG_DBLostField);
     return false;
 }				/* intoFields */
 
 static bool
-rowCountCheck(const char *action, int cnt1)
+rowCountCheck(int action, int cnt1)
 {
     int cnt2 = rv_lastNrows;
-    static char rowword1[] = "rows";
-    static char rowword2[] = "records";
 
     if(cnt1 == cnt2)
 	return true;
 
-    rowword1[3] = (cnt1 == 1 ? 0 : 's');
-    rowword2[6] = (cnt2 == 1 ? 0 : 's');
-    setError("oops!  I %s %d %s, and %d database %s were affected.",
-       action, cnt1, rowword1, cnt2, rowword2);
+    setError(MSG_DBDeleteCount + action, cnt1, cnt2);
     return false;
 }				/* rowCountCheck */
 
@@ -1022,7 +1010,7 @@ static int
 keyCountCheck(void)
 {
     if(!td->key1) {
-	setError("key column not specified");
+	setError(MSG_DBNoKeyCol);
 	return false;
     }
     return (td->key2 ? 2 : 1);
@@ -1035,43 +1023,43 @@ static const short insupdExceptions[] = { EXCSQLMISC,
 };
 
 static bool
-insupdError(const char *action, int rcnt)
+insupdError(int action, int rcnt)
 {
     int rc = rv_lastStatus;
-    const char *desc;
+    int msg;
 
     if(rc) {
 	switch (rc) {
 	case EXCVIEWUSE:
-	    desc = "cannot modify a view";
+	    msg = MSG_DBView;
 	    break;
 	case EXCREFINT:
-	    desc = "some other row in the database depends on this row";
+	    msg = MSG_DBRefInt;
 	    break;
 	case EXCITEMLOCK:
-	    desc = "row or table is locked";
+	    msg = MSG_DBLocked;
 	    break;
 	case EXCPERMISSION:
-	    desc =
-	       "you do not have permission to modify the database in this way";
+	    msg = MSG_DBPerms;
 	    break;
 	case EXCDEADLOCK:
-	    desc = "deadlock detected";
+	    msg = MSG_DBDeadlock;
 	    break;
 	case EXCNOTNULLCOLUMN:
-	    desc = "placing null into a not-null column";
+	    msg = MSG_DBNotNull;
 	    break;
 	case EXCCHECK:
-	    desc = "check constraint violated";
+	    msg = MSG_DBCheck;
 	    break;
 	case EXCTIMEOUT:
-	    desc = "daatabase timeout";
+	    msg = MSG_DBTimeout;
 	    break;
 	default:
-	    setError("miscelaneous sql error %d", rv_vendorStatus);
+	    setError(MSG_DBMisc, rv_vendorStatus);
 	    return false;
 	}
-	setError(desc);
+
+	setError(msg);
 	return false;
     }
 
@@ -1095,7 +1083,7 @@ sqlDelRows(int start, int end)
     ndel = end - start + 1;
     ln = start;
     if(ndel > 100) {
-	setError("cannot delete more than 100 rows at a time");
+	setError(MSG_DBMassDelete);
 	return false;
     }
 
@@ -1115,7 +1103,7 @@ sqlDelRows(int start, int end)
 	       td->name, td->cols[key1], lineFields[key1],
 	       td->cols[key2], lineFields[key2]);
 	nzFree(line);
-	if(!insupdError("deleted", 1))
+	if(!insupdError(0, 1))
 	    return false;
 	delText(ln, ln);
     }
@@ -1162,15 +1150,15 @@ sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 	l2 = strlen(lineFields[j]);
 	if(l1 != l2 || memcmp(s, lineFields[j], l1)) {
 	    if(j == key1 || j == key2) {
-		setError("cannot change a key column");
+		setError(MSG_DBChangeKey);
 		goto abort;
 	    }
 	    if(td->types[j] == 'B') {
-		setError("cannot change a blob field");
+		setError(MSG_DBChangeBlob);
 		goto abort;
 	    }
 	    if(td->types[j] == 'T') {
-		setError("cannot change a text field");
+		setError(MSG_DBChangeText);
 		goto abort;
 	    }
 	    if(*u1)
@@ -1194,7 +1182,7 @@ sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 	sql_exec("update %s set(%s) = (%s) where %s = %S and %s = %S",
 	   td->name, u1, u2,
 	   td->cols[key1], lineFields[key1], td->cols[key2], lineFields[key2]);
-    if(!insupdError("updated", 1))
+    if(!insupdError(2, 1))
 	goto abort;
 
     nzFree(d2);
@@ -1303,7 +1291,7 @@ sqlAddRows(int ln)
 	sql_exec("insert into %s (%s) values (%s)", td->name, u1, u2);
 	nzFree(u1);
 	nzFree(u2);
-	if(!insupdError("inserted", 1)) {
+	if(!insupdError(1, 1)) {
 	    printf("Error: ");
 	    showError();
 	    continue;
