@@ -102,8 +102,7 @@ loadBlacklist(void)
 		if(ipmask)
 		    ip &= ipmask;
 		if(nipblack == MAXIPBLACK)
-		    errorPrint("1too many ip addresses in blacklist, limit %d",
-		       MAXIPBLACK);
+		    i_printfExit(MSG_ManyIP, MAXIPBLACK);
 		ipblacklist[nipblack] = ip;
 		ipblackmask[nipblack] = ipmask;
 		ipblackcomp[nipblack] = (dotstop == '!');
@@ -168,7 +167,7 @@ getFileName(const char *defname, bool isnew)
     int l;
     char *p;
     while(true) {
-	printf("filename: ");
+	i_printf(MSG_FileName);
 	if(defname)
 	    printf("[%s] ", defname);
 	if(!fgets(buf, sizeof (buf), stdin))
@@ -187,7 +186,7 @@ getFileName(const char *defname, bool isnew)
 	} else
 	    defname = 0;
 	if(isnew && fileTypeByName(p, false)) {
-	    printf("Sorry, file %s already exists.\n", p);
+	    i_printf(MSG_FileExists, p);
 	    defname = 0;
 	    continue;
 	}
@@ -195,23 +194,30 @@ getFileName(const char *defname, bool isnew)
     }
 }				/* getFileName */
 
+static bool ignoreImages;
+
 static void
 writeAttachment(struct MHINFO *w)
 {
     const char *atname;
-    if(ismc && w->atimage)
+    if((ismc | ignoreImages) && w->atimage)
 	return;			/* image ignored */
     if(w->error64 == BAD64)
-	printf("abbreviated ");
+	i_printf(MSG_Abbreviated);
     if(w->start == w->end) {
-	printf("empty attachment");
+	i_printf(MSG_AttEmpty);
 	if(w->cfn[0])
 	    printf(" %s", w->cfn);
-	printf("\n");
+	nl();
 	atname = "x";
     } else {
-	printf("attachment ");
+	i_printf(MSG_Att);
 	atname = getFileName((w->cfn[0] ? w->cfn : 0), true);
+/* X is like x, but deletes all future images */
+	if(stringEqual(atname, "X")) {
+	    atname = "x";
+	    ignoreImages = true;
+	}
     }
     if(!ismc && stringEqual(atname, "e")) {
 	int cx, svcx = context;
@@ -219,12 +225,12 @@ writeAttachment(struct MHINFO *w)
 	    if(!sessionList[cx].lw)
 		break;
 	if(cx == MAXSESSION) {
-	    printf("no buffers left, atachment discarded");
+	    i_printf(MSG_AttNoBuffer);
 	} else {
 	    cxSwitch(cx, false);
-	    printf("session %d\n", cx);
+	    i_printf(MSG_SessionX, cx);
 	    if(!addTextToBuffer((pst) w->start, w->end - w->start, 0))
-		printf("could not copy the attachment into buffer %d", cx);
+		i_printf(MSG_AttNoCopy, cx);
 	    else if(w->cfn[0])
 		cw->fileName = cloneString(w->cfn);
 	    cxSwitch(svcx, false);	/* back to where we were */
@@ -232,13 +238,13 @@ writeAttachment(struct MHINFO *w)
     } else if(!stringEqual(atname, "x")) {
 	int fh = open(atname, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, 0666);
 	if(fh < 0) {
-	    printf("cannot create file %s, attachment is discarded\n", atname);
+	    i_printf(MSG_AttNoSave, atname);
 	    if(ismc)
 		exit(1);
 	} else {
 	    int nb = w->end - w->start;
 	    if(write(fh, w->start, nb) < nb) {
-		printf("cannot write to file %s, attachment is discarded\n");
+		i_printf(MSG_AttNoWrite, atname);
 		if(ismc)
 		    exit(1);
 	    }
@@ -279,14 +285,13 @@ fetchMail(int account)
 
     if(!isInteractive) {
 	if(!zapMail)
-	    errorPrint
-	       ("1fetchmail client is interactive, and cannot run in the background");
+	    i_printfExit(MSG_FetchNotBackgnd);
     }
 
     if(!mailDir)
-	errorPrint("1mailbox directory not specified in your .ebrc file");
+	i_printfExit(MSG_NoMailDir);
     if(chdir(mailDir))
-	errorPrint("1Cannot change directory to %s", mailDir);
+	i_printfExit(MSG_NoDirChange, mailDir);
 
     if(!loadAddressBook())
 	showErrorAbort();
@@ -296,7 +301,7 @@ fetchMail(int account)
     if(!serverGetLine())
 	showErrorAbort();
     if(memcmp(serverLine, "+OK ", 4))
-	errorPrint("1Unexpected pop3 introduction <%s>", serverLine);
+	i_printfExit(MSG_BadPopIntro, serverLine);
     sprintf(serverLine, "user %s%s", login, eol);
     serverPutGet(serverLine);
     if(pass) {			/* I think this is always required */
@@ -304,17 +309,15 @@ fetchMail(int account)
 	serverPutGet(serverLine);
     }				/* password */
     if(memcmp(serverLine, "+OK", 3))
-	errorPrint("1Could not complete the pop3 login/password sequence <%s>",
-	   serverLine);
+	i_printfExit(MSG_PopNotComplete, serverLine);
 
 /* How many mail messages? */
     serverPutGet("stat\r\n");
     if(memcmp(serverLine, "+OK ", 4))
-	errorPrint("1Could not obtain status information on your mailbox <%s>",
-	   serverLine);
+	i_printfExit(MSG_NoStatusMailBox, serverLine);
     nmsgs = atoi(serverLine + 4);
     if(!nmsgs) {
-	printf("no mail\n");
+	i_puts(MSG_NoMail);
 	serverClose();
 	exit(0);
     }
@@ -325,7 +328,7 @@ fetchMail(int account)
 /* the conect will drop when the program exits */
     }
 
-    printf("%d messages\n", nmsgs);
+    i_printf(MSG_MessagesX, nmsgs);
     if(zapMail && nmsgs > 300)
 	nmsgs = 300;
 
@@ -366,16 +369,13 @@ fetchMail(int account)
 	    while(true) {
 		int nr = tcp_read(mssock, retrbuf, sizeof (retrbuf));
 		if(nr <= 0)
-		    errorPrint
-		       ("1error reading message from the server, errno %d",
-		       errno);
+		    i_printfExit(MSG_ErrorReadMess, errno);
 		if(retr1) {
 /* add null, to make it easy to print the error message, if necessary */
 		    if(nr < sizeof (retrbuf))
 			retrbuf[nr] = 0;
 		    if(memcmp(retrbuf, "+OK", 3))
-			errorPrint("cannot fetch mail message %d <%s>", m,
-			   retrbuf);
+			i_printfExit(MSG_ErrorFetchMess, m, retrbuf);
 		    j = 3;
 		    while(retrbuf[j] != '\n')
 			++j;
@@ -430,28 +430,28 @@ fetchMail(int account)
 		    if(*redirect == '-')
 			++redirect, key = 'u';
 		    if(stringEqual(redirect, "x"))
-			puts("junk");
+			i_puts(MSG_Junk);
 		    else
 			printf("> %s\n", redirect);
 		} else if((mailIsSpam | mailIsBlack) && spamCan) {
 		    redirect = spamCan;
 		    key = 'u';
-		    printf("spam");
+		    i_printf(MSG_Spam);
 		    if(lastMailInfo->from[0]) {
-			printf(" from ");
+			i_printf(MSG_From);
 			printf("%s", lastMailInfo->from);
 		    } else if(lastMailInfo->reply[0]) {
-			printf(" from ");
+			i_printf(MSG_From);
 			printf("%s", lastMailInfo->reply);
 		    }
-		    printf("\n");
+		    nl();
 		} else if(!nattach &&	/* drop empty mail message */
 		   cw->dol -
 		   (lastMailInfo->subject != 0) -
 		   (lastMailInfo->from != 0) -
 		   (lastMailInfo->reply != 0) <= 1) {
 		    redirect = "x";
-		    printf("empty\n");
+		    i_puts(MSG_Empty);
 		}
 	    }
 	    if(redirect)
@@ -486,25 +486,25 @@ fetchMail(int account)
 
 			switch (key) {
 			case 'q':
-			    puts("quit");
+			    i_puts(MSG_Quit);
 			    serverClose();
 			case 'x':
 			    exit(0);
 			case 'n':
-			    puts("next");
+			    i_puts(MSG_Next);
 			    goto afterinput;
 			case 'd':
-			    puts("delete");
+			    i_puts(MSG_Delete);
 			    delflag = true;
 			    goto afterinput;
 			case 'i':
-			    puts("ip delete");
+			    i_puts(MSG_IPDelete);
 			    if(!cw->iplist || cw->iplist[0] == -1) {
 				if(passMail)
-				    puts("-p option disables this feature");
+				    i_puts(MSG_POption);
 				else
-				    puts(ipbFile ? "none" :
-				       "no blacklist file specified, feature disabled");
+				    i_puts(ipbFile ? MSG_None :
+				       MSG_NoBlackList);
 			    } else {
 				IP32bit addr;
 				for(k = 0; (addr = cw->iplist[k]) != NULL_IP;
@@ -522,34 +522,24 @@ fetchMail(int account)
 			    goto afterinput;
 			case 'j':
 			case 'J':
-			    puts("junk");
+			    i_puts(MSG_Junk);
 			    if(!junkSubject(lastMailInfo->subject, key))
 				continue;
 			    delflag = true;
 			    goto afterinput;
 			case ' ':
 			    if(displine > cw->dol)
-				puts("end of message");
+				i_puts(MSG_EndMessage);
 			    goto nextpage;
 			case '?':
-			    printf("?\tprint this help message.\n\
-q\tquit this program.\n\
-x\texit without changing anything on the mail server.\n\
-space\tread more of this mail message.\n\
-n\tmove on to the next mail message.\n\
-d\tdelete this message.\n\
-j\tjunk this subject for ten days.\n\
-J\tjunk this subject for a year.\n\
-w\twrite this message to a file and delete it.\n\
-k\tkeep this message in a file, but don't delete it.\n\
-u\twrite this message unformatted to a file, and delete it.\n");
+			    i_puts(MSG_MailHelp);
 			    continue;
 			case 'k':
 			case 'w':
 			case 'u':
 			    break;
 			default:
-			    puts("not yet implemented");
+			    i_puts(MSG_NYI);
 			    continue;
 			}	/* switch */
 		    }
@@ -569,7 +559,7 @@ u\twrite this message unformatted to a file, and delete it.\n");
 			   open(atname, O_WRONLY | O_TEXT | O_CREAT | O_APPEND,
 			   0666);
 			if(fh < 0) {
-			    printf("cannot create file %s\n", atname);
+			    i_printf(MSG_NoCreate, atname);
 			    goto savemail;
 			}
 			if(exists)
@@ -579,7 +569,7 @@ u\twrite this message unformatted to a file, and delete it.\n");
 			if(key == 'u' || unformatMail) {
 			    if(write(fh, exact, exact_l) < exact_l) {
 			      badsave:
-				printf("cannot write to file %s\n", atname);
+				i_printf(MSG_NoWrite, atname);
 				close(fh);
 				goto savemail;
 			    }
@@ -600,10 +590,10 @@ u\twrite this message unformatted to a file, and delete it.\n");
 				writeAttachments(lastMailInfo);
 			}	/* unformat or format */
 			if(atname != spamCan) {
-			    printf("mail save, %d bytes", fsize);
+			    i_printf(MSG_MailSaved, fsize);
 			    if(exists)
-				printf(" appended");
-			    printf("\n");
+				i_printf(MSG_Appended);
+			    nl();
 			}
 		    }		/* saving to a real file */
 		    goto afterinput;
@@ -621,9 +611,9 @@ u\twrite this message unformatted to a file, and delete it.\n");
 	    if(!serverPutLine(serverLine))
 		showErrorAbort();
 	    if(!serverGetLine())
-		errorPrint("1Sorry, you took too long; mail server hung up.");
+		i_printfExit(MSG_MailTimeOver);
 	    if(memcmp(serverLine, "+OK", 3))
-		errorPrint("1unable to delete message <%s>", serverLine);
+		i_printfExit(MSG_UnableDelMail, serverLine);
 	}
 	/* deleted */
 	nzFree(exact);
@@ -714,11 +704,16 @@ emailTest(void)
 	    continue;
 	if(*q++ != ':')
 	    continue;
-	for(n = 0; mhwords[n]; ++n)
-	    if(memEqualCI(mhwords[n], p, q - p))
-		break;
-	if(mhwords[n])
+/* X-Whatever is a mail header word */
+	if(q - p >= 8 && p[1] == '-' && toupper(p[0]) == 'X') {
 	    ++k;
+	} else {
+	    for(n = 0; mhwords[n]; ++n)
+		if(memEqualCI(mhwords[n], p, q - p))
+		    break;
+	    if(mhwords[n])
+		++k;
+	}
 	if(k >= 4 && k * 2 >= j)
 	    return true;
     }				/* loop over lines */
@@ -759,7 +754,7 @@ unpack64(struct MHINFO *w)
 	if(equals) {
 	    if(c == '=')
 		continue;
-	    runningError("unexpected characters after the encoded attachment");
+	    runningError(MSG_AttAfterChars);
 	    w->error64 = EXTRA64;
 	    break;
 	}
@@ -769,7 +764,7 @@ unpack64(struct MHINFO *w)
 	}
 	val = unb64(c);
 	if(val & 64) {
-	    runningError("invalid characters in the encoded attachment");
+	    runningError(MSG_AttBad64);
 	    w->error64 = BAD64;
 	    break;
 	}
@@ -817,6 +812,7 @@ unpackQP(struct MHINFO *w)
 	*r++ = '=';
     }
     w->end = r;
+    *r = 0;
 }				/* unpackQP */
 
 /* Look for the name of the attachment and boundary */
@@ -1123,7 +1119,7 @@ headerGlean(char *start, char *end)
     }
 
     if(debugLevel >= 5) {
-	printf("mail header analyzed\n");
+	puts("mail header analyzed");
 	printf("subject: %s\n", w->subject);
 	printf("from: %s\n", w->from);
 	printf("date: %s\n", w->date);
@@ -1523,7 +1519,7 @@ emailParse(char *buf)
     struct MHINFO *w, *v;
     nattach = nimages = 0;
     firstAttach = 0;
-    mailIsHtml = mailIsSpam = false;
+    mailIsHtml = mailIsSpam = ignoreImages = false;
     fm = initString(&fm_l);
     w = headerGlean(buf, buf + strlen(buf));
     mailIsHtml = (mailTextType(w) == CT_HTML);
