@@ -339,7 +339,7 @@ findEndScript(const char *h, const char *tagname,
     end = strstrCI(s, look);
     if(!end) {
 	rc = false;
-	browseError("no closing %s", look);
+	browseError(MSG_CloseTag, look);
 	end = (char *)h + strlen(h);
     } else if(is_js) {
 /* Check for document.write("</script>");
@@ -1042,7 +1042,7 @@ htmlReformat(const char *buf)
 	tagno = strtol(h + 1, (char **)&nh, 10);
 	c = *nh++;
 	if(!c || !strchr("{}<>*", c))
-	    errorPrint("@tag code %d has bad character %c following", tagno, c);
+	    i_printfExit(MSG_BadTagCode, tagno, c);
 	appendPrintableChunk(h, nh - h, premode);
 	preFormatCheck(tagno, &pretag, &slash);
 	if(pretag)
@@ -1132,74 +1132,92 @@ andTranslate(const char *s, bool invisible)
 	"percnt\0%",
 	"amp\0&",
 	"iexcl\0!",
-	"ntilde\0\xf1",
-	"Ntilde\0\xd1",
-	"agrave\0\xe0",
-	"Agrave\0\xc0",
-	"egrave\0\xe8",
-	"Egrave\0\xc8",
-	"igrave\0\xec",
-	"Igrave\0\xcc",
-	"ograve\0\xf2",
-	"Ograve\0\xd2",
-	"ugrave\0\xf9",
-	"Ugrave\0\xd9",
-	"auml\0\xe4",
-	"Auml\0\xc4",
-	"euml\0\xeb",
-	"Euml\0\xcb",
-	"iuml\0\xef",
-	"Iuml\0\xcf",
-	"ouml\0\xf6",
-	"Ouml\0\xd6",
-	"uuml\0\xfc",
-	"Uuml\0\xdc",
-	"yuml\0\xff",
-	"Yuml\0Y",
-	"aacute\0\xe1",
-	"Aacute\0\xc1",
-	"eacute\0\xe9",
-	"Eacute\0\xc9",
-	"iacute\0\xed",
-	"Iacute\0\xcd",
-	"oacute\0\xf3",
-	"Oacute\0\xd3",
-	"uacute\0\xfa",
-	"Uacute\0\xda",
-	"yacute\0\xfd",
-	"Yacute\0\xdd",
-	"atilde\0\xe3",
-	"Atilde\0\xc3",
-	"itilde\0i",
-	"Itilde\0I",
-	"otilde\0\xf5",
-	"Otilde\0\xd5",
-	"utilde\0u",
-	"Utilde\0U",
-	"acirc\0\xe2",
-	"Acirc\0\xc2",
-	"ecirc\0\xea",
-	"Ecirc\0\xca",
-	"icirc\0\xee",
-	"Icirc\0\xce",
-	"ocirc\0\xf4",
-	"Ocirc\0\xd4",
-	"ucirc\0\xfb",
-	"Ucirc\0\xdb",
-	"pound\0\xa3",
 	"cent\0\xa2",
-	"sdot\0\xb7",
-	"middot\0\xb7",
-	"edot\0e",
-	"nbsp\0 ",
-	"times\0\xd7",
-	"divide\0\xf7",
-	"deg\0\xb0",
+	"pound\0\xa3",
+	"yen\0\xa5",
+	"brvbar\0\xa6",
 	"copy\0\xa9",
 	"reg\0\xae",
+	"deg\0\xb0",
+	"plusmn\0\xb1",
+	"para\0\xb6",
+	"sdot\0\xb7",
+	"middot\0\xb7",
 	"frac14\0\xbc",
 	"half\0\xbd",
 	"frac34\0\xbe",
+	"iquest\0\xbf",
+	"Agrave\0\xc0",
+	"Aacute\0\xc1",
+	"Acirc\0\xc2",
+	"Atilde\0\xc3",
+	"Auml\0\xc4",
+	"Aring\0\xc5",
+	"AElig\0\xc6",
+	"Ccedil\0\xc7",
+	"Egrave\0\xc8",
+	"Eacute\0\xc9",
+	"Ecirc\0\xca",
+	"Euml\0\xcb",
+	"Igrave\0\xcc",
+	"Iacute\0\xcd",
+	"Icirc\0\xce",
+	"Iuml\0\xcf",
+	"ETH\0\xd0",
+	"Ntilde\0\xd1",
+	"Ograve\0\xd2",
+	"Oacute\0\xd3",
+	"Ocirc\0\xd4",
+	"Otilde\0\xd5",
+	"Ouml\0\xd6",
+	"times\0\xd7",
+	"Oslash\0\xd8",
+	"Ugrave\0\xd9",
+	"Uacute\0\xda",
+	"Ucirc\0\xdb",
+	"Uuml\0\xdc",
+	"Yacute\0\xdd",
+	"THORN\0\xde",
+	"szlig\0\xdf",
+	"agrave\0\xe0",
+	"aacute\0\xe1",
+	"acirc\0\xe2",
+	"atilde\0\xe3",
+	"auml\0\xe4",
+	"aring\0\xe5",
+	"aelig\0\xe6",
+	"ccedil\0\xe7",
+	"egrave\0\xe8",
+	"eacute\0\xe9",
+	"ecirc\0\xea",
+	"euml\0\xeb",
+	"igrave\0\xec",
+	"iacute\0\xed",
+	"icirc\0\xee",
+	"iuml\0\xef",
+	"eth\0\xf0",
+	"ntilde\0\xf1",
+	"ograve\0\xf2",
+	"oacute\0\xf3",
+	"ocirc\0\xf4",
+	"otilde\0\xf5",
+	"ouml\0\xf6",
+	"divide\0\xf7",
+	"oslash\0\xf8",
+	"ugrave\0\xf9",
+	"uacute\0\xfa",
+	"ucirc\0\xfb",
+	"uuml\0\xfc",
+	"yacute\0\xfd",
+	"thorn\0\xfe",
+	"yuml\0\xff",
+	"Yuml\0Y",
+	"itilde\0i",
+	"Itilde\0I",
+	"utilde\0u",
+	"Utilde\0U",
+	"edot\0e",
+	"nbsp\0 ",
 	"frac13\01/3",
 	"frac23\02/3",
 	"plusmn\0+-",
