@@ -168,7 +168,7 @@ static const struct tagInfo elements[] = {
     {"IMAGE", "an image", TAGACT_IMAGE, 0, 0, 4},
     {"BR", "a line break", TAGACT_BR, 0, 1, 4},
     {"P", "a paragraph", TAGACT_NOP, 0, 2, 5},
-    {"DIV", "a divided section", TAGACT_DIV, 1, 5, 0},
+    {"DIV", "a divided section", TAGACT_DIV, 3, 5, 0},
     {"HTML", "html", TAGACT_NOP, 0, 0, 0},
     {"BLOCKQUOTE", "a quoted paragraph", TAGACT_NOP, 1, 10, 1},
     {"H1", "a level 1 header", TAGACT_NOP, 1, 10, 1},
@@ -241,6 +241,7 @@ struct htmlTag {
     bool retain:1;
     bool multiple:1;
     bool rdonly:1;
+    bool clickable:1;		/* but not an input field */
     bool secure:1;
     bool checked:1;
     bool rchecked:1;		/* for reset */
@@ -346,7 +347,7 @@ get_js_event(const char *name)
     if(stringEqual(name, "onchange")) {
 	if(action == TAGACT_INPUT || action == TAGACT_SELECT) {
 	    if(itype == INP_TA)
-		runningError("onchange handler does not work with textarea");
+		runningError(MSG_OnchangeText);
 	    else if(itype > INP_HIDDEN && itype <= INP_SELECT) {
 		topTag->handler = true;
 		if(currentForm)
@@ -356,6 +357,8 @@ get_js_event(const char *name)
     }
 }				/* get_js_event */
 
+static bool strayClick;
+
 static void
 get_js_events(void)
 {
@@ -374,21 +377,19 @@ get_js_events(void)
 /* Some warnings about some handlers that we just don't "handle" */
     if(handlerPresent(ev, "onkeypress") ||
        handlerPresent(ev, "onkeyup") || handlerPresent(ev, "onkeydown"))
-	browseError
-	   ("warning, javascript cannot be invoked through keystroke events");
+	browseError(MSG_JSKeystroke);
     if(handlerPresent(ev, "onfocus") || handlerPresent(ev, "onblur"))
-	browseError("javascript cannot be invoked through focus or blur");
+	browseError(MSG_JSFocus);
     if(handlerPresent(ev, "ondblclick"))
-	runningError("warning, javascript cannot be invoked by a double click");
+	runningError(MSG_Doubleclick);
     if(handlerPresent(ev, "onclick")) {
 	if((action == TAGACT_A || action == TAGACT_AREA || action == TAGACT_FRAME) && topTag->href || action == TAGACT_INPUT && (itype <= INP_SUBMIT || itype >= INP_RADIO)) ;	/* ok */
 	else
-	    browseError
-	       ("warning, onclick code is not associated with a hyperlink or button");
+	    strayClick = true;
     }
     if(handlerPresent(ev, "onchange")) {
 	if(action != TAGACT_INPUT && action != TAGACT_SELECT || itype == INP_TA)
-	    browseError("onchange handler is not accessible");
+	    browseError(MSG_StrayOnchange);
     }
 /* Other warnings might be appropriate, but I'm going to assume this
  * is valid javascript, and you won't put an onsubmit function on <P> etc */
@@ -527,11 +528,11 @@ formControl(bool namecheck)
 	topTag->controller = currentForm;
 	fo = currentForm->jv;
     } else if(itype != INP_BUTTON)
-	browseError("%s is not part of a fill-out form", topTag->info->desc);
+	browseError(MSG_NotInForm2, topTag->info->desc);
 
     htmlName();
     if(namecheck && !topTag->name)
-	browseError("%s does not have a name", topTag->info->desc);
+	browseError(MSG_FieldNoName, topTag->info->desc);
 
     if(fo) {
 	topTag->jv = e =
@@ -601,7 +602,7 @@ htmlForm(void)
 	if(stringEqualCI(a, "post"))
 	    topTag->post = true;
 	else if(!stringEqualCI(a, "get"))
-	    browseError("unrecognized method, plese use GET or POST");
+	    browseError(MSG_GetPost);
 	nzFree(a);
     }
 
@@ -610,8 +611,7 @@ htmlForm(void)
 	if(stringEqualCI(a, "multipart/form-data"))
 	    topTag->mime = true;
 	else if(!stringEqualCI(a, "application/x-www-form-urlencoded"))
-	    browseError
-	       ("unrecognized enctype, plese use multipart/form-data or application/x-www-form-urlencoded");
+	    browseError(MSG_Enctype);
 	nzFree(a);
     }
 
@@ -625,7 +625,7 @@ htmlForm(void)
 	    else if(stringEqualCI(prot, "https"))
 		topTag->secure = true;
 	    else if(!stringEqualCI(prot, "http"))
-		browseError("form cannot submit using protocol %s", prot);
+		browseError(MSG_FormProtBad, prot);
 	}
     }
 
@@ -650,10 +650,9 @@ jsdw(void)
     memcpy(cw->dw + 3, "<html>\n", 7);
     side = sideBuffer(0, cw->dw + 10, -1, cw->fileName, true);
     if(side) {
-	printf("A separate window has been created in buffer %d\n", side);
+	i_printf(MSG_SideBufferX, side);
     } else {
-	puts
-	   ("The following has been sent to your browser, but there is no buffer to contain it.");
+	i_puts(MSG_NoSideBuffer);
 	printf("%s\n", cw->dw + 10);
     }
     nzFree(cw->dw);
@@ -671,7 +670,7 @@ htmlInput(void)
 	caseShift(s, 'l');
 	n = stringInList(inp_types, s);
 	if(n < 0) {
-	    browseError("unrecognized input type %s", s);
+	    browseError(MSG_InputType, s);
 	    n = INP_TEXT;
 	}
     }
@@ -701,7 +700,7 @@ htmlInput(void)
 	    }
 	    sprintf(namebuf, "|%s|", topTag->name);
 	    if(strstr(radioChecked, namebuf)) {
-		browseError("multiple radio buttons have been selected");
+		browseError(MSG_RadioMany);
 		return;
 	    }
 	    stringAndString(&radioChecked, &radioChecked_l, namebuf + 1);
@@ -850,15 +849,12 @@ locateOptions(const struct htmlTag *sel, const char *input,
 	    t = locateOptionByName(sel, iopt, &pmc, false);
 	if(!t) {
 	    if(n >= 0)
-		setError("%d is out of range", n);
+		setError(MSG_XOutOfRange, n);
 	    else
-		setError("%s options contain the string %s",
-		   (pmc ? "multiple" : "no"), iopt);
+		setError(pmc + MSG_OptMatchNone, iopt);
 /* This should never happen when we're doing a set check */
 	    if(setcheck) {
-		runningError
-		   ("error resolving option %s during sync or form submit",
-		   iopt);
+		runningError(MSG_OptionSync, iopt);
 		continue;
 	    }
 	    goto fail;
@@ -944,17 +940,18 @@ jSyncup(void)
 	    set_property_bool(eo, "checked", checked);
 	    continue;
 	}
-	/* checkable */
+
 	value = getFieldFromBuffer(t->seqno);
 /* If that line has been deleted from the user's buffer,
  * indicated by value = 0,
  * revert back to the original (reset) value. */
 
 	if(itype == INP_SELECT) {
-/* should value be replaced here, or do we know it's ok? */
 	    locateOptions(t, (value ? value : t->value), 0, 0, true);
+	    if(!t->multiple)
+		value = cloneString( get_property_option(eo));
 	}
-	/* select */
+
 	if(itype == INP_TA) {
 	    if(!value) {
 		set_property_string(eo, "value", 0);
@@ -972,7 +969,7 @@ jSyncup(void)
 	    nzFree(cxbuf);
 	    continue;
 	}
-	/* text area */
+
 	if(value) {
 	    set_property_string(eo, "value", value);
 	    nzFree(value);
@@ -1007,19 +1004,18 @@ findOpenTag(const char *name)
 	if(closing) {
 	    if(match)
 		return t;
-	    browseError("%s is closed inside %s", desc, t->info->desc);
+	    browseError(MSG_TagNest, desc, t->info->desc);
 	    continue;
 	}
 	if(!match)
 	    continue;
 	if(!(t->info->nest & 2))
-	    browseError("%s begins in the middle of %s", desc, desc);
+	    browseError(MSG_TagInTag, desc, desc);
 	return t;
     }				/* loop */
 
     if(closing)
-	browseError("an unexpected closure of %s, which was never opened",
-	   desc);
+	browseError(MSG_TagClose, desc);
     return NULL;
 }				/* findOpenTag */
 
@@ -1040,6 +1036,8 @@ newTag(const char *name)
     t->info = ti;
     t->seqno = ntags++;
     t->balanced = true;
+    if(stringEqual(name, "a"))
+	t->clickable = true;
     addToListBack(&htmlStack, t);
     return t;
 }				/* newTag */
@@ -1056,8 +1054,7 @@ onloadGo(void *obj, const char *jsrc, const char *tagname)
     if(!handlerPresent(obj, "onunload"))
 	return;
     if(handlerPresent(obj, "onclick")) {
-	runningError
-	   ("tag cannot have onunload and onclik handlers simultaneously");
+	runningError(MSG_UnloadClick);
 	return;
     }
 
@@ -1236,7 +1233,7 @@ encodeTags(char *html)
 	    currentTA = 0;
 	    if(slash)
 		continue;
-	    browseError("a text area begins in the middle of a text area");
+	    browseError(MSG_TextareaNest);
 	}
 
 	if(!action)
@@ -1273,14 +1270,19 @@ encodeTags(char *html)
 	    continue;		/* negated version means nothing */
 
 /* Does this tag force the closure of an anchor? */
-	if(ti->bits & TAG_CLOSEA && currentA &&
-	   (action != TAGACT_A || !slash) &&
-	   (a_text || action <= TAGACT_OPTION)) {
-	    browseError("%s appears inside an anchor", ti->desc);
-	    stringAndString(&new, &l, "\2000}");
-	    currentA->balanced = true;
-	    currentA = 0;
-/* when the </a> comes along, it will be unbalanced, and we'll ignore it. */
+	if(currentA && (action != TAGACT_A || !slash)) {
+	    if(open && open->clickable)
+		goto forceCloseAnchor;
+	    rc = htmlAttrPresent(topAttrib, "onclick");
+	    if(rc ||
+	       ti->bits & TAG_CLOSEA && (a_text || action <= TAGACT_OPTION)) {
+		browseError(MSG_InAnchor, ti->desc);
+	      forceCloseAnchor:
+		stringAndString(&new, &l, "\2000}");
+		currentA->balanced = true;
+		currentA = 0;
+/* if/when the </a> comes along, it will be unbalanced, and we'll ignore it. */
+	    }
 	}
 
 	retainTag = true;
@@ -1291,6 +1293,8 @@ encodeTags(char *html)
 	if(ti->bits & TAG_INVISIBLE)
 	    invisible = !slash;
 
+	strayClick = false;
+
 /* Are we gathering text to build title or option? */
 	if(currentTitle || currentOpt) {
 	    char **ptr;
@@ -1299,7 +1303,7 @@ encodeTags(char *html)
 	    if(v->action != action &&
 	       !(v->action == TAGACT_OPTION && action == TAGACT_SELECT) ||
 	       action != TAGACT_OPTION && !slash)
-		browseError("%s contains html tags", v->info->desc);
+		browseError(MSG_HasTags, v->info->desc);
 	    if(!(ti->bits & TAG_CLOSEA))
 		continue;
 /* close off the title or option */
@@ -1318,8 +1322,7 @@ encodeTags(char *html)
 		    for(y = a; *y; ++y)
 			if(*y == ',')
 			    *y = ' ';
-		    browseError
-		       ("option cannot contain a comma when it is part of a multiple select");
+		    browseError(MSG_OptionComma);
 		}
 		spaceCrunch(a, true, true);
 		*ptr = a;
@@ -1331,7 +1334,7 @@ encodeTags(char *html)
 
 		if(currentOpt) {
 		    if(!*a) {
-			browseError("empty option");
+			browseError(MSG_OptionEmpty);
 		    } else {
 			if(!v->value)
 			    v->value = cloneString(a);
@@ -1392,7 +1395,7 @@ encodeTags(char *html)
 	    if(slash)
 		continue;
 	    if(cw->ft)
-		browseError("multiple titles");
+		browseError(MSG_ManyTitles);
 	    offset = l;
 	    currentTitle = t;
 	    continue;
@@ -1410,8 +1413,10 @@ encodeTags(char *html)
 		   domLink("Anchor", topTag->name, topTag->id, "href",
 		   topTag->href, "links", jdoc, false);
 		get_js_events();
-		if(t->href)
+		if(t->href) {
 		    a_href = true;
+		    topTag->clickable = true;
+		}
 		a_text = false;
 	    }
 	    if(a_href) {
@@ -1485,7 +1490,7 @@ encodeTags(char *html)
 		}
 	    }
 	    if(j < 0)
-		browseError("%s is not inside a list", ti->desc);
+		browseError(MSG_NotInList, ti->desc);
 	    if(!retainTag)
 		continue;
 	    hnum[0] = '\r';
@@ -1508,7 +1513,7 @@ encodeTags(char *html)
 		break;
 	}
 	    if(v == (struct htmlTag *)&htmlStack)
-		browseError("%s is not inside a list", ti->desc);
+		browseError(MSG_NotInList, ti->desc);
 	    goto nop;
 
 	case TAGACT_TABLE:
@@ -1530,27 +1535,27 @@ encodeTags(char *html)
 
 	case TAGACT_TR:
 	    if(!intable) {
-		browseError("%s is not inside a table", ti->desc);
+		browseError(MSG_NotInTable, ti->desc);
 		continue;
 	    }
-/* This really doesn't work for tables inside tables */
 	    if(slash)
 		--inrow;
 	    else
 		++inrow;
 	    tdfirst = true;
-	    if((!slash) && (open = findOpenTag("table")) && (to = open->jv)) {
+	    if((!slash) && (open = findOpenTag("table")) && open->jv) {
 		htmlName();
-		topTag->jv =
+		topTag->jv = to =
 		   domLink("Trow", topTag->name, topTag->id, 0, 0,
 		   "rows", open->jv, false);
 		get_js_events();
+		establish_property_array(to, "cells");
 	    }
 	    goto nop;
 
 	case TAGACT_TD:
 	    if(!inrow) {
-		browseError("%s is not inside a table row", ti->desc);
+		browseError(MSG_NotInRow, ti->desc);
 		continue;
 	    }
 	    if(slash)
@@ -1563,6 +1568,13 @@ encodeTags(char *html)
 		new[l] = 0;
 		stringAndChar(&new, &l, '|');
 	    }
+	    if((open = findOpenTag("tr")) && open->jv) {
+		htmlName();
+		topTag->jv = to =
+		   domLink("Cell", topTag->name, topTag->id, 0, 0,
+		   "cells", open->jv, false);
+		get_js_events();
+	    }
 	    goto endtag;
 
 	case TAGACT_DIV:
@@ -1599,7 +1611,7 @@ encodeTags(char *html)
 	    else
 		j &= 3;
 	    if(!j)
-		continue;
+		goto endtag;
 	    c = '\f';
 	    if(j == 1) {
 		c = '\r';
@@ -1609,7 +1621,7 @@ encodeTags(char *html)
 	    if(currentA)
 		c = ' ';
 	    stringAndChar(&new, &l, c);
-	    continue;
+	    goto endtag;
 
 	case TAGACT_FORM:
 	    htmlForm();
@@ -1664,7 +1676,7 @@ encodeTags(char *html)
 	    if(slash)
 		continue;
 	    if(!currentSel) {
-		browseError("option appears outside a select statement");
+		browseError(MSG_NotInSelect);
 		continue;
 	    }
 	    currentOpt = t;
@@ -1674,7 +1686,7 @@ encodeTags(char *html)
 	    t->value = htmlAttrVal(topAttrib, "value");
 	    if(htmlAttrPresent(topAttrib, "selected")) {
 		if(currentSel->lic && !currentSel->multiple)
-		    browseError("multiple options are selected");
+		    browseError(MSG_ManyOptSelected);
 		else
 		    t->checked = t->rchecked = true, ++currentSel->lic;
 	    }
@@ -1738,6 +1750,7 @@ encodeTags(char *html)
 		   domLink("Area", topTag->name, topTag->id, "href",
 		   topTag->href, "areas", jdoc, false);
 	    }
+	    topTag->clickable = true;
 	    get_js_events();
 	    if(!retainTag)
 		continue;
@@ -1824,8 +1837,7 @@ encodeTags(char *html)
 /* I'm not going to process an open ended script. */
 	    if(!rc) {
 		nzFree(javatext);
-		runningError
-		   ("script is not closed at eof, suspending javascript");
+		runningError(MSG_ScriptNotClosed);
 		cw->jsdead = true;
 		continue;
 	    }
@@ -1846,9 +1858,7 @@ encodeTags(char *html)
 			if(browseLocal && !isURL(t->href)) {
 			    if(!fileIntoMemory(t->href, &serverData,
 			       &serverDataLen)) {
-				runningError
-				   ("could not fetch local javascript, %s",
-				   errorMsg);
+				runningError(MSG_GetLocalJS, errorMsg);
 			    } else {
 				javatext = serverData;
 				prepareForBrowse(javatext, serverDataLen);
@@ -1859,20 +1869,19 @@ encodeTags(char *html)
 				prepareForBrowse(javatext, serverDataLen);
 			    } else {
 				nzFree(serverData);
-				runningError
-				   ("could not fetch javascript from %s, code %d",
-				   t->href, hcode);
+				runningError(MSG_GetJS, t->href, hcode);
 			    }
 			} else {
-			    runningError("could not fetch javascript, %s",
-			       errorMsg);
+			    runningError(MSG_GetJS2, errorMsg);
 			}
 			js_line = 1;
 			js_file = t->href;
 		    }
 		}		/* fetch from the net */
 		if(javatext) {
-		    char *w = strrchr(js_file, '/');
+		    char *w = 0;
+		    if(js_file)
+			w = strrchr(js_file, '/');
 		    if(w) {
 /* Trailing slash doesn't count */
 			if(w[1] == 0 && w > js_file)
@@ -1925,7 +1934,7 @@ encodeTags(char *html)
 	    continue;
 
 	default:
-	    browseError("unprocessed tag action %d", action);
+	    browseError(MSG_BadTag, action);
 	    continue;
 	}			/* switch */
 
@@ -1942,6 +1951,14 @@ encodeTags(char *html)
 	stringAndString(&new, &l, hnum);
       endtag:
 	lastact = action;
+	if(strayClick) {
+	    topTag->clickable = true;
+	    a_text = false;
+	    topTag->href = cloneString("#");
+	    currentA = topTag;
+	    sprintf(hnum, "%c%d{", InternalCodeChar, topTag->seqno);
+	    stringAndString(&new, &l, hnum);
+	}
     }				/* loop over html string */
 
     if(currentA) {
@@ -1989,7 +2006,7 @@ encodeTags(char *html)
 	foreach(t, htmlStack) {
 	    browseLine = t->ln;
 	    if(t->info->nest && !t->slash && !t->balanced) {
-		browseError("%s is not closed at eof", t->info->desc);
+		browseError(MSG_TagNotClosed, t->info->desc);
 		break;
 	    }
 
@@ -2013,7 +2030,7 @@ encodeTags(char *html)
 		    break;
 	    }
 	    if(v == (struct htmlTag *)&htmlStack) {	/* end of list */
-		browseError("label %s not found", a);
+		browseError(MSG_NoLable2, a);
 		break;
 	    }
 	}			/* loop over all tags */
@@ -2044,7 +2061,7 @@ preFormatCheck(int tagno, bool * pretag, bool * slash)
 {
     const struct htmlTag *t;
     if(!parsePage)
-	errorPrint("@calling preFormatTag without parsePage");
+	i_printfExit(MSG_ErrCallPreFormat);
     *pretag = *slash = false;
     if(tagno >= 0 && tagno < ntags) {
 	t = tagArray[tagno];
@@ -2059,7 +2076,7 @@ htmlParse(char *buf, int remote)
     char *newbuf;
 
     if(parsePage)
-	errorPrint("@htmlParse() is not reentrant.");
+	i_printfExit(MSG_HtmlNotreentrant);
     parsePage = true;
     if(remote >= 0)
 	browseLocal = !remote;
@@ -2356,7 +2373,7 @@ infShow(int tagno, const char *search)
 	++s;
     printf("%s", s);
     if(t->multiple)
-	printf(" many");
+	i_printf(MSG_Many);
     if(t->itype >= INP_TEXT && t->itype <= INP_NUMBER && t->lic)
 	printf("[%d]", t->lic);
     if(t->itype == INP_TA) {
@@ -2366,8 +2383,8 @@ infShow(int tagno, const char *search)
 	if(rows && cols) {
 	    printf("[%sx%s", rows, cols);
 	    if(wrap && stringEqualCI(wrap, "virtual"))
-		printf(" recommended");
-	    printf("]");
+		i_printf(MSG_Recommended);
+	    i_printf(MSG_Close);
 	}
 	nzFree(rows);
 	nzFree(cols);
@@ -2375,7 +2392,7 @@ infShow(int tagno, const char *search)
     }				/* text area */
     if(t->name)
 	printf(" %s", t->name);
-    printf("\n");
+    nl();
     if(t->itype != INP_SELECT)
 	return;
 
@@ -2396,9 +2413,9 @@ infShow(int tagno, const char *search)
     }
     if(!show) {
 	if(!search)
-	    puts("no options");
+	    i_puts(MSG_NoOptions);
 	else
-	    printf("no options contain the string %s\n", search);
+	    i_printf(MSG_NoOptionsMatch, search);
     }
 }				/* infShow */
 
@@ -2415,44 +2432,42 @@ infReplace(int tagno, const char *newtext, int notify)
 
 /* sanity checks on the input */
     if(itype <= INP_SUBMIT) {
-	char *b = "";
+	int b = MSG_IsButton;
 	if(itype == INP_SUBMIT || itype == INP_IMAGE)
-	    b = "submit ";
+	    b = MSG_SubmitButton;
 	if(itype == INP_RESET)
-	    b = "reset ";
-	setError("this is a %sbutton; use i* to push the button", b);
+	    b = MSG_ResetButton;
+	setError(b);
 	return false;
     }
 
     if(itype == INP_TA) {
-	setError("this is a textarea, you must edit it from session %d",
-	   t->lic);
+	setError(MSG_Textarea, t->lic);
 	return false;
     }
 
     if(t->rdonly) {
-	setError("readonly field");
+	setError(MSG_Readonly);
 	return false;
     }
 
     if(strchr(newtext, '\n')) {
-	setError("input field cannot contain a newline character");
+	setError(MSG_InputNewline);
 	return false;
     }
 
     if(itype >= INP_TEXT && itype <= INP_NUMBER && t->lic && newlen > t->lic) {
-	setError("input too long, limit %d", t->lic);
+	setError(MSG_InputLong, t->lic);
 	return false;
     }
 
     if(itype >= INP_RADIO) {
 	if(newtext[0] != '+' && newtext[0] != '-' || newtext[1]) {
-	    setError("field must be set to + or -");
+	    setError(MSG_InputRadio);
 	    return false;
 	}
 	if(itype == INP_RADIO && newtext[0] == '-') {
-	    setError
-	       ("you cannot clear a radio button; you must set another one");
+	    setError(MSG_ClearRadio);
 	    return false;
 	}
     }
@@ -2475,14 +2490,14 @@ infReplace(int tagno, const char *newtext, int notify)
 	if(!envFile(newtext, &newtext))
 	    return false;
 	if(newtext[0] && access(newtext, 4)) {
-	    setError("%s is not an accessible file", newtext);
+	    setError(MSG_FileAccess, newtext);
 	    return false;
 	}
     }
 
     if(itype == INP_NUMBER) {
 	if(*newtext && stringIsNum(newtext) < 0) {
-	    setError("number expected");
+	    setError(MSG_NumberExpected);
 	    return false;
 	}
     }
@@ -2509,7 +2524,7 @@ infReplace(int tagno, const char *newtext, int notify)
 
     if(itype >= INP_RADIO && tagHandler(t->seqno, "onclick")) {
 	if(cw->jsdead)
-	    runningError("javascript disabled, skipping the onclick code");
+	    runningError(MSG_NJNoOnclick);
 	else {
 	    jSyncup();
 	    handlerGo(t->jv, "onclick");
@@ -2522,7 +2537,7 @@ infReplace(int tagno, const char *newtext, int notify)
     if(itype >= INP_TEXT && itype <= INP_SELECT &&
        tagHandler(t->seqno, "onchange")) {
 	if(cw->jsdead)
-	    runningError("javascript disabled, skipping the onchange code");
+	    runningError(MSG_NJNoOnchange);
 	else {
 	    jSyncup();
 	    handlerGo(t->jv, "onchange");
@@ -2622,7 +2637,7 @@ formReset(const struct htmlTag *form)
 	    set_property_number(t->jv, "selectedIndex", -1);
     }				/* loop over tags */
 
-    puts("form has been reset");
+    i_puts(MSG_FormReset);
 }				/* formReset */
 
 /* Fetch a field value (from a form) to post. */
@@ -2672,6 +2687,7 @@ postDelimiter(char fsep, const char *boundary, char **post, int *l)
     if(fsep == '-') {
 	stringAndString(post, l, "--");
 	stringAndString(post, l, boundary);
+	stringAndChar(post, l, '\r');
 	fsep = '\n';
     }
     stringAndChar(post, l, fsep);
@@ -2702,7 +2718,7 @@ postNameVal(const char *name, const char *val,
 	    break;
 	case '\n':
 	    stringAndString(post, l, name);
-	    stringAndString(post, l, "=\n");
+	    stringAndString(post, l, "=\r\n");
 	    break;
 	case '-':
 	    stringAndString(post, l, "Content-Disposition: form-data; name=\"");
@@ -2727,7 +2743,7 @@ postNameVal(const char *name, const char *val,
 	break;
     case '\n':
 	stringAndString(post, l, val);
-	stringAndChar(post, l, '\n');
+	stringAndString(post, l, eol);
 	break;
     case '-':
 	if(isfile) {
@@ -2751,13 +2767,13 @@ postNameVal(const char *name, const char *val,
 		    break;
 		}
 	}
-	stringAndString(post, l, "\nContent-Type: ");
+	stringAndString(post, l, "\r\nContent-Type: ");
 	stringAndString(post, l, ct);
-	stringAndString(post, l, "\nContent-Transfer-Encoding: ");
+	stringAndString(post, l, "\r\nContent-Transfer-Encoding: ");
 	stringAndString(post, l, ce);
-	stringAndString(post, l, "\n\n");
+	stringAndString(post, l, "\r\n\r\n");
 	stringAndString(post, l, val);
-	stringAndChar(post, l, '\n');
+	stringAndString(post, l, eol);
 	if(isfile)
 	    nzFree(enc);
 	break;
@@ -2786,7 +2802,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 	boundary = makeBoundary();
 	stringAndString(post, l, "`mfd~");
 	stringAndString(post, l, boundary);
-	stringAndChar(post, l, '\n');
+	stringAndString(post, l, eol);
     }
 
     while(t = *list++) {
@@ -2868,7 +2884,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 		    goto fail;
 		for(j = 0; j < cxlen; ++j)
 		    if(cxbuf[j] == 0) {
-			setError("session %d contains nulls", cx);
+			setError(MSG_SessionNull, cx);
 			nzFree(cxbuf);
 			goto fail;
 		    }
@@ -2934,8 +2950,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 	    if(!*value)
 		continue;
 	    if(!(form->post & form->mime)) {
-		setError
-		   ("sending a file requires method=post and enctype=multipart/form-data");
+		setError(MSG_FilePost);
 		nzFree(value);
 		goto fail;
 	    }
@@ -2946,7 +2961,7 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
 	    goto fail;
 	}
 	/* file */
-	errorPrint("@unexpected input type in submitForm()");
+	i_printfExit(MSG_UnexSubmitForm);
 
       success:
 	postNameVal(name, value, fsep, false, boundary, post, l);
@@ -2955,12 +2970,12 @@ formSubmit(const struct htmlTag *form, const struct htmlTag *submit,
     if(form->mime) {		/* the last boundary */
 	stringAndString(post, l, "--");
 	stringAndString(post, l, boundary);
-	stringAndString(post, l, "--\n");
+	stringAndString(post, l, "--\r\n");
     }
 
     if(noname)
-	debugPrint(0, "warning: fields without names will not be transmitted");
-    puts("submitting form");
+	i_puts(MSG_UnnamedFields);
+    i_puts(MSG_FormSubmit);
     return true;
 
   fail:
@@ -3004,20 +3019,19 @@ infPush(int tagno, char **post_string)
     }
 
     if(itype > INP_SUBMIT) {
-	setError("this is an input field, not a button");
+	setError(MSG_NoButton);
 	return false;
     }
 
     if(!form && itype != INP_BUTTON) {
-	setError("this button is not part of a form");
+	setError(MSG_NotInForm);
 	return false;
     }
 
     if(t && tagHandler(t->seqno, "onclick")) {
 	if(cw->jsdead)
-	    runningError("javascript disabled, %s",
-	       itype ==
-	       INP_BUTTON ? "no action taken" : "skipping the onclick code");
+	    runningError(itype ==
+	       INP_BUTTON ? MSG_NJNoAction : MSG_NJNoOnclick);
 	else {
 	    rc = handlerGo(t->jv, "onclick");
 	    jsdw();
@@ -3030,7 +3044,7 @@ infPush(int tagno, char **post_string)
 
     if(itype == INP_BUTTON) {
 	if(!handlerPresent(t->jv, "onclick")) {
-	    setError("no javascript associated with this button");
+	    setError(MSG_ButtonNoJS);
 	    return false;
 	}
 	return true;
@@ -3040,7 +3054,7 @@ infPush(int tagno, char **post_string)
 /* Before we reset, run the onreset code */
 	if(t && tagHandler(form->seqno, "onreset")) {
 	    if(cw->jsdead)
-		runningError("javascript disabled, skipping the onreset code");
+		runningError(MSG_NJNoReset);
 	    else {
 		rc = handlerGo(form->jv, "onreset");
 		jsdw();
@@ -3057,7 +3071,7 @@ infPush(int tagno, char **post_string)
     /* Before we submit, run the onsubmit code */
     if(t && tagHandler(form->seqno, "onsubmit")) {
 	if(cw->jsdead)
-	    runningError("javascript disabled, skipping the onsubmit code");
+	    runningError(MSG_NJNoSubmit);
 	else {
 	    rc = handlerGo(form->jv, "onsubmit");
 	    jsdw();
@@ -3086,7 +3100,7 @@ infPush(int tagno, char **post_string)
     }
 
     if(!action) {
-	setError("form does not include a destination url");
+	setError(MSG_FormNoURL);
 	return false;
     }
 
@@ -3094,13 +3108,13 @@ infPush(int tagno, char **post_string)
 
     prot = getProtURL(action);
     if(!prot) {
-	setError("the action of the form is not a url");
+	setError(MSG_FormBadURL);
 	return false;
     }
 
     if(stringEqualCI(prot, "javascript")) {
 	if(cw->jsdead) {
-	    setError("javascript is disabled, cannot activate this form");
+	    setError(MSG_NJNoForm);
 	    return false;
 	}
 	javaParseExecute(form->jv, action, 0, 0);
@@ -3115,12 +3129,11 @@ infPush(int tagno, char **post_string)
 	form->bymail = true;
     } else if(stringEqualCI(prot, "http")) {
 	if(form->secure) {
-	    setError
-	       ("This form has changed from https to http, and is now insecure");
+	    setError(MSG_BecameInsecure);
 	    return false;
 	}
     } else if(!stringEqualCI(prot, "https")) {
-	setError("cannot submit using protocol %s", prot);
+	setError(MSG_SubmitProtBad, prot);
 	return false;
     }
 
@@ -3128,17 +3141,15 @@ infPush(int tagno, char **post_string)
     stringAndString(&post, &l, action);
     section = strchr(post, '#');
     if(section) {
-	debugPrint(0,
-	   "warning: form url specifies a section %s, which will be ignored",
-	   section);
+	i_printf(MSG_SectionIgnored, section);
 	*section = 0;
 	l = strlen(post);
     }
     section = strpbrk(post, "?\1");
     if(section) {
 	if(*section == '\1' || !(form->bymail | form->post)) {
-	    debugPrint(0,
-	       "warning: the url already specifies some data, which will be overwritten by the data in this form");
+	    debugPrint(3,
+	       "the url already specifies some data, which will be overwritten by the data in this form");
 	    *section = 0;
 	    l = strlen(post);
 	}
@@ -3178,12 +3189,11 @@ infPush(int tagno, char **post_string)
 	    sprintf(q, "subject:html form(%s)\n", name ? name : "?");
 	strcpy(q + strlen(q), post + actlen);
 	nzFree(post);
-	printf("sending mail to %s\n", addr);
+	i_printf(MSG_MailSending, addr);
 	sleep(1);
 	rc = sendMail(localAccount, tolist, q, -1, atlist, 0, false);
 	if(rc)
-	    printf
-	       ("Your information has been sent; watch for a reply via return mail.\n");
+	    i_puts(MSG_MailSent);
 	nzFree(addr);
 	nzFree(subj);
 	nzFree(q);
@@ -3202,13 +3212,12 @@ tagFromJavaVar(void *v)
     struct htmlTag **list = cw->tags;
     struct htmlTag *t;
     if(!list)
-	errorPrint("@list = 0 in tagFromJavaVar()");
+	i_printfExit(MSG_NullListInform);
     while(t = *list++)
 	if(t->jv == v)
 	    break;
     if(!t)
-	runningError
-	   ("could not find the html tag associated with the javascript variable being modified");
+	runningError(MSG_LostTag);
     return t;
 }				/* tagFromJavaVar */
 
@@ -3226,8 +3235,7 @@ javaSetsTagVar(void *v, const char *val)
 	return;
     }
     if(t->itype == INP_TA) {
-	runningError
-	   ("javascript modified a textarea, and that isn't implemented yet");
+	runningError(MSG_JSTextarea);
 	return;
     }
     updateFieldInBuffer(t->seqno, val, parsePage ? 0 : 2, false);
@@ -3267,7 +3275,7 @@ javaOpensWindow(const char *href, const char *name)
     const char *a;
 
     if(!href || !*href) {
-	browseError("java is opening a blank window");
+	browseError(MSG_JSBlankWindow);
 	return;
     }
 
@@ -3292,7 +3300,7 @@ javaOpensWindow(const char *href, const char *name)
 void
 javaSetsTimeout(int n, const char *jsrc, void *to, bool isInterval)
 {
-    struct htmlTag *t = newTag("A");
+    struct htmlTag *t = newTag("a");
     char timedesc[48];
     int l;
 
