@@ -59,8 +59,8 @@ ssl_init(bool doConfig)
     if(!sslCerts) {
 	verifyCertificates = false;
 	if(doConfig)
-	    debugPrint(1,
-	       "no ssl certificate file specified; secure connections cannot be verified");
+	    if(debugLevel >= 1)
+		i_puts(MSG_NoCertFile);
     }
 
     SSL_CTX_set_default_verify_paths(sslcx);
@@ -115,8 +115,7 @@ readSocket(SSL * ssl, int fh)
 	else
 	    n = tcp_readFully(fh, p->data, CHUNKSIZE);
 	if(n < 0) {
-	    setError(intFlag ? opint :
-	       "could not read the data from the server");
+	    setError(intFlag ? MSG_Interrupted : MSG_WebRead);
 	    free(p);
 	    for(p = chunklist; p; p = q) {
 		q = p->next;
@@ -140,7 +139,7 @@ readSocket(SSL * ssl, int fh)
 	isprintByte = true;
     }				/* loop reading data */
     if(isprintByte)
-	printf("\n");
+	nl();
 
 /* Put it all together */
     serverData = data = allocMem(len + 1);
@@ -220,8 +219,8 @@ extractHeaderParam(const char *str, const char *item)
 time_t
 parseHeaderDate(const char *date)
 {
-    static const char *const months[12] =
-       { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+    static const char *const months[12] = {
+	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
     };
     time_t t = 0;
@@ -401,7 +400,7 @@ parseRefresh(char *ref, int *delay_p)
 	*delay_p = delay;
 	return true;
     }
-    debugPrint(0, "warning: garbled refresh directive, %s", ref);
+    i_printf(MSG_GarbledRefresh, ref);
     *delay_p = 0;
     return false;
 }				/* parseRefresh */
@@ -414,7 +413,7 @@ refreshDelay(int sec, const char *u)
 /* the value 15 seconds is somewhat arbitrary */
     if(sec < 15)
 	return true;
-    printf("redirect to %s delayed by %d seconds", u, sec);
+    i_printf(MSG_RedirectDelayed, u, sec);
     return false;
 }				/* refreshDelay */
 
@@ -446,7 +445,7 @@ httpConnect(const char *from, const char *url)
     char suffix[12];
 
     if(!isURL(url)) {
-	setError("%s is not a url", url);
+	setError(MSG_BadURL, url);
 	return false;
     }
 
@@ -470,10 +469,10 @@ httpConnect(const char *from, const char *url)
     secure = stringEqualCI(prot, "https");
     host = getHostURL(url);
     if(!host)
-	errorPrint("@empty host in httpConnect");
+	i_printfExit(MSG_EmptyHost);
     if(proxy_host) {
 	if(secure) {
-	    setError("secure proxy not yet implemented");
+	    setError(MSG_SSLProxyNYI);
 	    return false;
 	}
 	hip = tcp_name_ip(proxy_host);
@@ -481,7 +480,7 @@ httpConnect(const char *from, const char *url)
 	hip = tcp_name_ip(host);
     }
     if(hip == -1) {
-	setError(intFlag ? opint : "cannot identify %s on the network", host);
+	setError((intFlag ? MSG_Interrupted : MSG_IdentifyHost), host);
 	return false;
     }
     debugPrint(4, "%s -> %s",
@@ -499,9 +498,7 @@ httpConnect(const char *from, const char *url)
 	nzFree(cmd);
 	return true;
     } else {
-	setError
-	   ("the %s protocol is not supported by edbrowse, and is not included in the mime types in your config file",
-	   prot);
+	setError(MSG_WebProtBad, prot);
 	return false;
     }
 
@@ -524,8 +521,7 @@ httpConnect(const char *from, const char *url)
 	s = getUserURL(url);
 	if(s) {
 	    if(strlen(s) >= sizeof (user) - 2) {
-		setError("user name too long, limit %d characters",
-		   sizeof (user));
+		setError(MSG_UserNameLong, sizeof (user));
 		return false;
 	    }
 	    strcpy(user, s);
@@ -533,8 +529,7 @@ httpConnect(const char *from, const char *url)
 	s = getPassURL(url);
 	if(s) {
 	    if(strlen(s) >= sizeof (pass) - 2) {
-		setError("password too long, limit %d characters",
-		   sizeof (pass));
+		setError(MSG_PasswordLong, sizeof (pass));
 		return false;
 	    }
 	    strcpy(pass, s);
@@ -550,7 +545,7 @@ httpConnect(const char *from, const char *url)
     getPortLocURL(url, &portloc, &port);
     hsock = tcp_connect(hip, (proxy_host ? proxy_port : port), webTimeout);
     if(hsock < 0) {
-	setError(intFlag ? opint : "cannot connect to %s", host);
+	setError((intFlag ? MSG_Interrupted : MSG_WebConnect), host);
 	return false;
     }
     if(proxy_host)
@@ -568,12 +563,9 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	    err = ERR_peek_last_error();
 	    ERR_clear_error();
 	    if(ERR_GET_REASON(err) != SSL_R_CERTIFICATE_VERIFY_FAILED)
-		setError("cannot establish a secure connection to %s, error %d",
-		   host, err);
+		setError(MSG_WebConnectSecure, host, err);
 	    else
-		setError
-		   ("The certificate for host %s could not be verified - SSL connection aborted",
-		   host);
+		setError(MSG_NoCertify, host);
 	    SSL_free(hssl);
 	    close(hsock);
 	    return false;
@@ -673,9 +665,9 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	    post += 5;
 	    stringAndString(&hdr, &l,
 	       "Content-Type: multipart/form-data; boundary=");
-	    s = strchr(post, '\n');
+	    s = strchr(post, '\r');
 	    stringAndBytes(&hdr, &l, post, s - post);
-	    post = s + 1;
+	    post = s + 2;
 	    stringAndString(&hdr, &l, eol);
 	}
 	stringAndString(&hdr, &l, "Content-Length: ");
@@ -707,8 +699,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
     debugPrint(4, "http header sent, %d/%d bytes", n, l);
     free(hdr);
     if(n < l) {
-	setError(intFlag ? opint :
-	   "could not send the request to the web server");
+	setError(intFlag ? MSG_Interrupted : MSG_WebSend);
 	if(secure)
 	    SSL_free(hssl);
 	close(hsock);
@@ -801,7 +792,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	int authmeth = 1;	/* basic method by default */
 	if(u = extractHeaderItem(serverData, hdr, "WWW-Authenticate", 0)) {
 	    if(!memEqualCI(u, "basic", 5) || isalnumByte(u[5])) {
-		setError("authorization method %s not recognized", u);
+		setError(MSG_BadAuthMethod, u);
 		nzFree(u);
 		goto abort;
 	    }
@@ -824,43 +815,41 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	}
 	if(!(user[0] | pass[0])) {
 	    if(!isInteractive) {
-		setError("web page requires authorization");
+		setError(MSG_Authorize2);
 		goto abort;
 	    }
-	    puts("authorization requested - type x to abort.");
+	    i_puts(MSG_WebAuthorize);
 	  getlogin:
-	    printf("Username: ");
+	    i_printf(MSG_UserName);
 	    fflush(stdout);
 	    fflush(stdin);
 	    if(!fgets(user, sizeof (user), stdin))
 		ebClose(0);
 	    n = strlen(user);
 	    if(n >= sizeof (user) - 1) {
-		printf("user name too long, limit %d characters",
-		   sizeof (user) - 2);
+		i_printf(MSG_UserNameLong, sizeof (user) - 2);
 		goto getlogin;
 	    }
 	    if(n && user[n - 1] == '\n')
 		user[--n] = 0;
 	    if(stringEqual(user, "x")) {
-		setError("login aborted");
+		setError(MSG_LoginAbort);
 		goto abort;
 	    }
-	    printf("Password: ");
+	    i_printf(MSG_Password);
 	    fflush(stdout);
 	    fflush(stdin);
 	    if(!fgets(pass, sizeof (pass), stdin))
 		ebClose(0);
 	    n = strlen(pass);
 	    if(n >= sizeof (pass) - 1) {
-		printf("user name too long, limit %d characters",
-		   sizeof (pass) - 2);
+		i_printf(MSG_PasswordLong, sizeof (pass) - 2);
 		goto getlogin;
 	    }
 	    if(n && pass[n - 1] == '\n')
 		pass[--n] = 0;
 	    if(stringEqual(pass, "x")) {
-		setError("login aborted");
+		setError(MSG_LoginAbort);
 		goto abort;
 	    }
 	}
@@ -884,8 +873,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	if(stringEqualCI(u, "8bit"))
 	    hce = 0;
 	if(hce < 0) {
-	    debugPrint(0, "warning: unrecognized http compression method %s",
-	       u);
+	    i_printf(MSG_CompressUnrecognized, u);
 	    hce = 0;
 	}
 	nzFree(u);
@@ -920,9 +908,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	    if(!loc[0])
 		loc = 0;
 	    if(!loc) {
-		debugPrint(0,
-		   "warning: http redirection %d, but a new url is not specified",
-		   hcode);
+		i_printf(MSG_RedirectNoURL, hcode);
 		if(hcode >= 301 && hcode <= 303)
 		    hcode = 200;
 	    } else
@@ -943,7 +929,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
 		}
 		if(recount >= 10) {
 		    free(u);
-		    puts("too many redirections");
+		    i_puts(MSG_RedirectMany);
 		    goto gotFile;
 		}
 /* Redirection looks valid, let's run with it. */
@@ -976,7 +962,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
     }
     /* looking for redirection */
     if(hcode != 200)
-	debugPrint(0, "warning, html error %d, %s", hcode, herror);
+	i_printf(MSG_HTTPError, hcode, herror);
 
   gotFile:
 /* Don't need the header any more */
@@ -1003,14 +989,12 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	   open(edbrowseTempFile, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC,
 	   0666);
 	if(fh < 0) {
-	    setError("cannot create temp file %s, to uncompress the web page",
-	       edbrowseTempFile);
+	    setError(MSG_TempNoCreate, edbrowseTempFile);
 	    *u = 0;
 	    goto abort;
 	}
 	if(write(fh, serverData, serverDataLen) < serverDataLen) {
-	    setError("cannot write to temp file %s, to uncompress the web page",
-	       edbrowseTempFile);
+	    setError(MSG_TempNoWrite, edbrowseTempFile);
 	    close(fh);
 	    *u = 0;
 	    goto abort;
@@ -1028,20 +1012,17 @@ hssl->options |= SSL_OP_NO_TLSv1;
 	free(scmd);
 	n = fileSizeByName(edbrowseTempFile);
 	if(n <= 0) {
-	    setError
-	       ("zcat cannot uncompress the data, to reconstruct the web page");
+	    setError(MSG_TempNoUncompress);
 	    return false;
 	}
 	serverData = allocMem(n + 2);
 	fh = open(edbrowseTempFile, O_RDONLY | O_BINARY);
 	if(fh < 0) {
-	    setError("cannot access the uncompressed web page in %s",
-	       edbrowseTempFile);
+	    setError(MSG_TempNoAccess, edbrowseTempFile);
 	    goto abort;
 	}
 	if(read(fh, serverData, n) < n) {
-	    setError("cannot read the uncompressed web page in %s",
-	       edbrowseTempFile);
+	    setError(MSG_TempNoRead, edbrowseTempFile);
 	    close(fh);
 	    goto abort;
 	}
@@ -1058,7 +1039,7 @@ hssl->options |= SSL_OP_NO_TLSv1;
     return true;
 
   nohead:
-    debugPrint(0, "warning: page does not have a recognizable http header");
+    i_puts(MSG_HTTPHeader);
     return true;
 
   abort:
@@ -1146,28 +1127,15 @@ ftpConnect(const char *url)
     int c;
     static const char npf[] = "not a plain file.";
     const int npfsize = strlen(npf);
-    static const char *const failmessages[] = { 0,
-	"could not connect to remote host",
-	"could not connect to remote host (timed out)",
-	"transfer failed",
-	"transfer failed (timed out)",
-	"no such directory",
-	"could not change directory (timed out)",
-	"malformed url",
-	"usage error",
-	"error in login config file",
-	"library initialization failed",
-	"session initialization failed",
-    };
 
     serverData = 0;
     serverDataLen = 0;
     fileSize = -1;
-    debugPrint(1, "ftp download");
+    if(debugLevel >= 1)
+	i_puts(MSG_FTPDownload);
     dirmode = false;
 
-  top:
-    cmd = initString(&cmd_l);
+  top:cmd = initString(&cmd_l);
     if(dirmode) {
 	stringAndString(&cmd, &cmd_l, "ncftpls -l ");
     } else {
@@ -1191,7 +1159,7 @@ ftpConnect(const char *url)
 
     f = popen(cmd, "r");
     if(!f) {
-	setError("could not spawn subcommand %s, errno %d", cmd, errno);
+	setError(MSG_TempNoSystem, cmd, errno);
 	nzFree(cmd);
 	return false;
     }
@@ -1239,12 +1207,12 @@ ftpConnect(const char *url)
 	if(!(rc & 0xff))
 	    rc >>= 8;
 	if(rc > 0 && rc <= 11)
-	    setError(failmessages[rc]);
+	    setError(MSG_FTPConnect - 1 + rc);
 	else
-	    setError("unexpected ftp error %d", rc);
+	    setError(MSG_FTPUnexpected, rc);
 	return false;
     }
-    puts(dirmode ? "directory" : "success");
+    i_puts(dirmode + MSG_Success);
     if(dirmode) {		/* need a final slash */
 	int l = strlen(url);
 	changeFileName = allocMem(l + 2);
@@ -1316,7 +1284,8 @@ allIPs(void)
 			    }
 			}
 		    }		/* valid ip */
-		}		/* different domain */
+		}
+		/* different domain */
 	    }			/* valid domain */
 	    nzFree(href);
 	    if(iptotal == 5)
