@@ -37,6 +37,11 @@ my_ErrorReporter(JSContext * cx, const char *message, JSErrorReport * report)
 {
     char *prefix, *tmp;
 
+    if(debugLevel < 2)
+	return;
+    if(ismc)
+	return;
+
     if(!report) {
 	fprintf(gErrFile, "%s\n", message);
 	return;
@@ -44,7 +49,7 @@ my_ErrorReporter(JSContext * cx, const char *message, JSErrorReport * report)
 
 /* Conditionally ignore reported warnings. */
     if(JSREPORT_IS_WARNING(report->flags)) {
-	if(ismc | browseLocal)
+	if(browseLocal)
 	    return;
     }
 
@@ -54,20 +59,23 @@ my_ErrorReporter(JSContext * cx, const char *message, JSErrorReport * report)
     if(report->lineno) {
 	tmp = prefix;
 	prefix = JS_smprintf("%s%u: ", tmp ? tmp : "", report->lineno);
-	JS_free(cx, tmp);
+	if(tmp)
+	    JS_free(cx, tmp);
     }
     if(JSREPORT_IS_WARNING(report->flags)) {
 	tmp = prefix;
 	prefix = JS_smprintf("%s%swarning: ",
 	   tmp ? tmp : "", JSREPORT_IS_STRICT(report->flags) ? "strict " : "");
-	JS_free(cx, tmp);
+	if(tmp)
+	    JS_free(cx, tmp);
     }
 
     if(prefix)
 	fputs(prefix, gErrFile);
     fprintf(gErrFile, "%s\n", message);
 
-    JS_free(cx, prefix);
+    if(prefix)
+	JS_free(cx, prefix);
 }				/* my_ErrorReporter */
 
 
@@ -188,7 +196,7 @@ win_close(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
    jsval * rval)
 {
 /* It's too confusing to just close the window */
-    puts("this page is finished, please use your back key or quit");
+    i_puts(MSG_PageDone);
     cw->jsdead = true;
     return JS_TRUE;
 }				/* win_close */
@@ -463,7 +471,7 @@ setter_innerText(JSContext * cx, JSObject * obj, jsval id, jsval * vp)
     if(!JSVAL_IS_STRING(v))
 	return JS_FALSE;
     s = JS_GetStringBytes(JSVAL_TO_STRING(v));
-    puts("Sorry, innerText update not yet implemented.");
+    i_puts(MSG_InnerText);
 /* The string has already been updated in the object. */
     return JS_TRUE;
 }				/* setter_innerText */
@@ -609,6 +617,13 @@ static JSClass trow_class = {
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
+static JSClass cell_class = {
+    "Cell",
+    JSCLASS_HAS_PRIVATE,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
+};
+
 static JSClass div_class = {
     "Div",
     JSCLASS_HAS_PRIVATE,
@@ -659,6 +674,7 @@ static struct DOMCLASS domClasses[] = {
     {&area_class},
     {&span_class},
     {&trow_class},
+    {&cell_class},
     {&option_class},
     {0}
 };
@@ -729,21 +745,21 @@ createJavaContext(void)
 /* 4 meg js space - should this be configurable? */
 	jrt = JS_NewRuntime(4L * 1024L * 1024L);
 	if(!jrt)
-	    errorPrint("2could not allocate memory for javascript operations.");
+	    i_printfExit(MSG_JavaMemError);
 	gOutFile = stdout;
 	gErrFile = stderr;
     }
 
     jcx = JS_NewContext(jrt, gStackChunkSize);
     if(!jcx)
-	errorPrint("2unable to create javascript context");
+	i_printfExit(MSG_JavaContextError);
     JS_SetErrorReporter(jcx, my_ErrorReporter);
     JS_SetOptions(jcx, JSOPTION_VAROBJFIX);
 
 /* Create the Window object, which is the global object in DOM. */
     jwin = JS_NewObject(jcx, &window_class, NULL, NULL);
     if(!jwin)
-	errorPrint("2unable to create window object for javascript");
+	i_printfExit(MSG_JavaWindowError);
     JS_InitClass(jcx, jwin, 0, &window_class, window_ctor, 3,
        NULL, window_methods, NULL, NULL);
 /* Ok, but the global object was created before the class,
@@ -752,7 +768,7 @@ createJavaContext(void)
 
 /* Math, Date, Number, String, etc */
     if(!JS_InitStandardClasses(jcx, jwin))
-	errorPrint("2unable to create standard classes for javascript");
+	i_printfExit(MSG_JavaClassError);
 
     establish_property_object(jwin, "window", jwin);
     establish_property_object(jwin, "self", jwin);
@@ -787,7 +803,7 @@ createJavaContext(void)
        NULL, doc_methods, NULL, NULL);
     jdoc = JS_NewObject(jcx, &doc_class, NULL, jwin);
     if(!jdoc)
-	errorPrint("2unable to create document object for javascript");
+	i_printfExit(MSG_JavaObjError);
     establish_property_object(jwin, "document", jdoc);
 
     establish_property_string(jdoc, "bgcolor", "white", false);
@@ -972,6 +988,10 @@ javaParseExecute(void *this, const char *str, const char *filename, int lineno)
     bool rc;
     jsval rval;
 
+/* Sometimes Mac puts these three chars at the start of a text file. */
+    if(!strncmp(str, "\xef\xbb\xbf", 3))
+	str += 3;
+
     debugPrint(6, "javascript:\n%s", str);
     ok = JS_EvaluateScript(jcx, this, str, strlen(str),
        filename, lineno, &rval);
