@@ -211,7 +211,7 @@ build_host(int exception, const char *hostname, int port)
     else
 	strcpy(urlbuffer, hostname);
     if(strlen(urlbuffer) >= sizeof (urlbuffer))
-	errorPrint("@hostname:port is too long");
+	i_printfExit(MSG_PortTooLong);
     v = STRING_TO_JSVAL(JS_NewStringCopyZ(jcx, urlbuffer));
     JS_SetProperty(jcx, uo, "host", &v);
     setter_suspend = false;
@@ -434,7 +434,7 @@ url_initialize(const char *url, bool readonly, bool exclude_href)
     if(s = getProtURL(url)) {
 	sprintf(urlbuffer, "%s:", s);
 	if(strlen(urlbuffer) >= sizeof (urlbuffer))
-	    errorPrint("@protocol: is too long");
+	    i_printfExit(MSG_ProtTooLong);
 	s = urlbuffer;
     }
     loc_def_set("protocol", s, setter_loc_prot, attr);
@@ -458,7 +458,7 @@ url_initialize(const char *url, bool readonly, bool exclude_href)
 	if(pl)
 	    sprintf(urlbuffer + strlen(urlbuffer), ":%d", port);
 	if(strlen(urlbuffer) >= sizeof (urlbuffer))
-	    errorPrint("@hostname:port is too long");
+	    i_printfExit(MSG_PortTooLong);
 	s = urlbuffer;
     }
     loc_def_set("host", s, setter_loc_host, attr);
@@ -837,6 +837,28 @@ get_property_bool(void *jv, const char *name)
     return JSVAL_TO_BOOLEAN(v);
 }				/* get_property_bool */
 
+const char *
+get_property_option(void *jv)
+{
+    JSObject *obj = jv;
+    jsval v;
+    JSObject *oa;		/* option array */
+    JSObject *oo;		/* option object */
+    int n;
+
+    if(!obj)
+	return 0;
+    JS_GetProperty(jcx, obj, "selectedIndex", &v);
+    n = JSVAL_TO_INT(v);
+    if(n < 0)
+	return 0;
+    JS_GetProperty(jcx, obj, "options", &v);
+    oa = JSVAL_TO_OBJECT(v);
+    JS_GetElement(jcx, oa, n, &v);
+    oo = JSVAL_TO_OBJECT(v);
+    return get_property_string(oo, "value");
+}				/* get_property_option */
+
 
 /*********************************************************************
 Manage the array of options under an html select.
