@@ -9,6 +9,8 @@
 
 #include <time.h>
 
+#define MAXRECAT 100		/* max number of recipients or attachments */
+
 char serverLine[MAXTTYLINE];
 static char spareLine[MAXTTYLINE];
 int mssock;			/* server socket */
@@ -53,7 +55,7 @@ loadAddressBook(void)
 	}
 	if(c == ':') {		/* delimiter */
 	    if(state == 0) {
-		setError("missing alias in address book line %d", ln);
+		setError(MSG_ABNoAlias, ln);
 	      freefail:
 		nzFree(buf);
 		return false;
@@ -76,7 +78,7 @@ loadAddressBook(void)
 	    if(state == 0)
 		continue;
 	    if(state == 1) {
-		setError("missing : in address book line %d", ln - 1);
+		setError(MSG_ABNoColon, ln - 1);
 		goto freefail;
 	    }
 	    if(state == 3) {
@@ -86,36 +88,26 @@ loadAddressBook(void)
 		*t = 0;
 		v = strchr(last, ':');
 		if(v - last >= 16) {
-		    setError
-		       ("alias in address book line %d is too long, limit 15 characters",
-		       ln - 1);
+		    setError(MSG_ABAliasLong, ln - 1);
 		    goto freefail;
 		}
 		++v;
 		if(t - v >= 64) {
-		    setError
-		       ("email in address book line %d is too long, limit 63 characters",
-		       ln - 1);
+		    setError(MSG_ABMailLong, ln - 1);
 		    goto freefail;
 		}
 		if(!strchr(v, '@')) {
-		    setError
-		       (" email in address book line %d does not have an @",
-		       ln - 1);
+		    setError(MSG_ABNoAt, ln - 1);
 		    goto freefail;
 		}
 		if(strpbrk(v, " \t")) {
-		    setError
-		       ("cannot handle whitespace in email, address book line %d",
-		       ln - 1);
+		    setError(MSG_ABMailSpaces, ln - 1);
 		    goto freefail;
 		}
 
 		while(last < t) {
 		    if(!isprintByte(*last)) {
-			setError
-			   ("unprintable characters in your alias or email, address book line %d",
-			   ln - 1);
+			setError(MSG_AbMailUnprintable, ln - 1);
 			goto freefail;
 		    }
 		    ++last;
@@ -139,7 +131,7 @@ loadAddressBook(void)
 
     *t = 0;
     if(state) {
-	setError("last line of your address book is not terminated");
+	setError(MSG_ABUnterminated);
 	goto freefail;
     }
 
@@ -197,7 +189,7 @@ serverPutLine(const char *buf)
     }
     n = tcp_write(mssock, buf, len);
     if(n < len) {
-	setError("cannot send data to the mail server");
+	setError(MSG_MailWrite);
 	return false;
     }
     return true;
@@ -216,7 +208,7 @@ serverGetLine(void)
 	len =
 	   tcp_read(mssock, serverLine + slen, sizeof (serverLine) - 1 - slen);
 	if(len <= 0) {
-	    setError("cannot read data from the mail server");
+	    setError(MSG_MailRead);
 	    return false;
 	}
 	slen += len;
@@ -224,7 +216,7 @@ serverGetLine(void)
     }
     s = strchr(serverLine, '\n');
     if(!s) {
-	setError("line from the mail server is too long, or unterminated");
+	setError(MSG_MailResponseLong);
 	return false;
     }
     strcpy(spareLine, s + 1);
@@ -250,13 +242,13 @@ mailConnect(const char *host, int port)
 {
     IP32bit ip = tcp_name_ip(host);
     if(ip == NULL_IP) {
-	setError(intFlag ? opint : "cannot locate the mail server %s", host);
+	setError((intFlag ? MSG_Interrupted : MSG_MailLocate), host);
 	return false;
     }
     debugPrint(4, "%s -> %s", host, tcp_ip_dots(ip));
     mssock = tcp_connect(ip, port, mailTimeout);
     if(mssock < 0) {
-	setError(intFlag ? opint : "cannot connect to the mail server");
+	setError(intFlag ? MSG_Interrupted : MSG_MailConnect);
 	return false;
     }
     debugPrint(4, "connected to port %d", port);
@@ -345,7 +337,7 @@ encodeAttachment(const char *file, int ismail,
 	    if(!unfoldBuffer(cx, false, &buf, &buflen))
 		return false;
 	    if(!buflen) {
-		setError("buffer %d is empty", cx);
+		setError(MSG_BufferXEmpty, cx);
 		goto freefail;
 	    }
 	    sprintf(newfilename, "<buffer %d>", cx);
@@ -356,7 +348,7 @@ encodeAttachment(const char *file, int ismail,
 	    if(!fileIntoMemory(file, &buf, &buflen))
 		return false;
 	    if(!buflen) {
-		setError("file %s is empty", file);
+		setError(MSG_FileXEmpty, file);
 		goto freefail;
 	    }
 	}
@@ -378,7 +370,7 @@ encodeAttachment(const char *file, int ismail,
 	while(*s == ' ' || *s == '\t')
 	    ++s;
 	if(!memEqualCI(s, "subject:", 8)) {
-	    setError("your email should begin with subject:");
+	    setError(MSG_SubjectStart);
 	    goto freefail;
 	}
 	s += 8;
@@ -391,12 +383,11 @@ encodeAttachment(const char *file, int ismail,
 	while(s > t && isspaceByte(s[-1]))
 	    --s;
 	if(s == t) {
-	    setError("empty subject line");
+	    setError(MSG_SubjectEmpty);
 	    goto freefail;
 	}
 	if(s - t >= sizeof (subjectLine)) {
-	    setError("subject line too long, limit %d characters",
-	       sizeof (subjectLine) - 1);
+	    setError(MSG_SubjectLong, sizeof (subjectLine) - 1);
 	    goto freefail;
 	}
 	memcpy(subjectLine, t, s - t);
@@ -405,8 +396,7 @@ encodeAttachment(const char *file, int ismail,
 	for(s = subjectLine; s < t; ++s) {
 	    c = *s;
 	    if(!isprintByte(c) && c != ' ') {
-		setError
-		   ("invalid characters in the subject line, please use only spaces and printable ascii text");
+		setError(MSG_SubjectChars);
 		goto freefail;
 	    }
 	}
@@ -425,7 +415,7 @@ encodeAttachment(const char *file, int ismail,
 	    if(c != 0) {
 		int fd, n;
 		if(c != 'f') {
-		    setError(".signature is not a regular file");
+		    setError(MSG_SigRegular);
 		    goto freefail;
 		}
 		n = fileSizeByName(sigFile);
@@ -433,7 +423,7 @@ encodeAttachment(const char *file, int ismail,
 		    buf = reallocMem(buf, buflen + n + 1);
 		    fd = open(sigFile, O_RDONLY);
 		    if(fd < 0) {
-			setError("cannot access .signature file");
+			setError(MSG_SigAccess);
 			goto freefail;
 		    }
 		    read(fd, buf + buflen, n);
@@ -494,9 +484,7 @@ encodeAttachment(const char *file, int ismail,
 
     if(buflen > 20 && nacount * 5 > buflen) {	/* binary file */
 	if(ismail) {
-	    setError
-	       ("cannot mail the binary file %s, perhaps this should be an attachment?",
-	       file);
+	    setError(MSG_MailBinary, file);
 	    goto freefail;
 	}
 
@@ -672,10 +660,12 @@ sendMail(int account, const char **recipients, const char *body,
     const char *from, *reply, *login, *smlogin, *pass;
     const struct MACCOUNT *a;
     const char *s, *boundary;
+    char reccc[MAXRECAT];
     char *t;
     int nat, cx, i, j;
     char *out = 0;
     bool mustmime = false;
+    bool firstrec;
     const char *ct, *ce;
     char *encoded = 0;
     struct MACCOUNT *localMail;
@@ -704,14 +694,26 @@ sendMail(int account, const char **recipients, const char *body,
     if(nat)
 	mustmime = true;
     if(nalt && nalt < nat) {
-	setError
-	   ("either none or all of the attachments must be declared \"alternative\"");
+	setError(MSG_AttAlternate);
 	return false;
     }
 
     if(!loadAddressBook())
 	return false;
 
+/* set copy flags */
+    for(j = 0; s = recipients[j]; ++j) {
+	char cc = 0;
+	if(*s == '^' || *s == '?')
+	    cc = *s++;
+	if(j == MAXRECAT) {
+	    setError(MSG_RecipMany, MAXRECAT);
+	    return false;
+	}
+	recipients[j] = s;
+	reccc[j] = cc;
+    }
+
 /* Look up aliases in the address book */
     for(j = 0; s = recipients[j]; ++j) {
 	if(strchr(s, '@'))
@@ -732,15 +734,15 @@ sendMail(int account, const char **recipients, const char *body,
 	    continue;
 	}
 	if(!addressFile) {
-	    setError
-	       ("No address book specified, please check your .ebrc config file");
+	    setError(MSG_ABMissing);
 	    return false;
 	}
-	setError("alias %s not found in your address book", s);
+	setError(MSG_ABNoAlias2, s);
 	return false;
     }				/* recipients */
+
     if(!j) {
-	setError("no recipients specified");
+	setError(MSG_RecipNone);
 	return false;
     }
 
@@ -750,21 +752,21 @@ sendMail(int account, const char **recipients, const char *body,
 	    if(!cxCompare(cx) || !cxActive(cx))
 		return false;
 	    if(!sessionList[cx].lw->dol) {
-		setError("session %d is empty, cannot atach", cx);
+		setError(MSG_AttSessionEmpty, cx);
 		return false;
 	    }
 	} else {
 	    char ftype = fileTypeByName(s, false);
 	    if(!ftype) {
-		setError("cannot access attachment %s", s);
+		setError(MSG_AttAccess, s);
 		return false;
 	    }
 	    if(ftype != 'f') {
-		setError("file %s is not a regular file, cannot attach", s);
+		setError(MSG_AttRegular, s);
 		return false;
 	    }
 	    if(!fileSizeByName(s)) {
-		setError("file %s is empty, cannot attach", s);
+		setError(MSG_AttEmpty2, s);
 		return false;
 	    }
 	}
@@ -788,9 +790,7 @@ sendMail(int account, const char **recipients, const char *body,
 	    goto mailfail;
     }
     if(!memEqualCI(serverLine, "220 ", 4)) {
-	setError
-	   ("unexpected prompt \"%s\" at the start of the sendmail session",
-	   serverLine);
+	setError(MSG_MailBadPrompt, serverLine);
 	goto mailfail;
     }
 
@@ -800,7 +800,7 @@ sendMail(int account, const char **recipients, const char *body,
     if(!serverGetLine())
 	goto mailfail;
     if(!memEqualCI(serverLine, "250 ", 4)) {
-	setError("mail server doesn't recognize %s", login);
+	setError(MSG_MailWhat, login);
 	goto mailfail;
     }
 
@@ -810,7 +810,7 @@ sendMail(int account, const char **recipients, const char *body,
     if(!serverGetLine())
 	goto mailfail;
     if(!memEqualCI(serverLine, "250 ", 4)) {
-	setError("mail server rejected %s", reply);
+	setError(MSG_MailReject, reply);
 	goto mailfail;
     }
 
@@ -821,7 +821,7 @@ sendMail(int account, const char **recipients, const char *body,
 	if(!serverGetLine())
 	    goto mailfail;
 	if(!memEqualCI(serverLine, "250 ", 4)) {
-	    setError("mail server rejected %s", s);
+	    setError(MSG_MailReject, s);
 	    goto mailfail;
 	}
     }
@@ -831,19 +831,46 @@ sendMail(int account, const char **recipients, const char *body,
     if(!serverGetLine())
 	goto mailfail;
     if(!memEqualCI(serverLine, "354 ", 4)) {
-	setError("mail server is not ready to receive data, %s", serverLine);
+	setError(MSG_MailNotReady, serverLine);
 	goto mailfail;
     }
 
 /* Build the outgoing mail, and send it in one go, as one string. */
     out = initString(&j);
+
+    firstrec = true;
     for(i = 0; s = recipients[i]; ++i) {
-	stringAndString(&out, &j, i ? "  " : "To: ");
+	if(reccc[i])
+	    continue;
+	stringAndString(&out, &j, firstrec ? "To:" : ",\r\n  ");
 	stringAndString(&out, &j, s);
-	if(recipients[i + 1])
-	    stringAndChar(&out, &j, ',');
-	stringAndString(&out, &j, eol);
+	firstrec = false;
     }
+    if(!firstrec)
+	stringAndString(&out, &j, eol);
+
+    firstrec = true;
+    for(i = 0; s = recipients[i]; ++i) {
+	if(reccc[i] != '^')
+	    continue;
+	stringAndString(&out, &j, firstrec ? "CC:" : ",\r\n  ");
+	stringAndString(&out, &j, s);
+	firstrec = false;
+    }
+    if(!firstrec)
+	stringAndString(&out, &j, eol);
+
+    firstrec = true;
+    for(i = 0; s = recipients[i]; ++i) {
+	if(reccc[i] != '?')
+	    continue;
+	stringAndString(&out, &j, firstrec ? "BCC:" : ",\r\n  ");
+	stringAndString(&out, &j, s);
+	firstrec = false;
+    }
+    if(!firstrec)
+	stringAndString(&out, &j, eol);
+
     sprintf(serverLine, "From: %s <%s>%s", from, reply, eol);
     stringAndString(&out, &j, serverLine);
     sprintf(serverLine, "Reply-to: %s <%s>%s", from, reply, eol);
@@ -920,7 +947,7 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 /* do these next two lines make any sense? */
        !strstrCI(serverLine, "message accepted") &&
        !strstrCI(serverLine, "message received")) {
-	setError("could not send mail message, %s", serverLine);
+	setError(MSG_MailNotSent, serverLine);
 	goto mailfail;
     }
 
@@ -938,18 +965,16 @@ bool
 validAccount(int n)
 {
     if(!maxAccount) {
-	setError("no mail accounts specified, plese check your config file");
+	setError(MSG_MailAccountsNone);
 	return false;
     }
     if(n <= 0 || n > maxAccount) {
-	setError("invalid account number %d, please use 1 through %d", n,
-	   maxAccount);
+	setError(MSG_MailAccountBad, n, maxAccount);
 	return false;
     }
     return true;
 }				/* validAccount */
 
-#define MAXRECAT 100		/* max number of recipients or attachments */
 bool
 sendMailCurrent(int sm_account, bool dosig)
 {
@@ -967,23 +992,23 @@ sendMailCurrent(int sm_account, bool dosig)
     bool subj = false;
 
     if(cw->browseMode) {
-	setError("cannot send mail while in browse mode");
+	setError(MSG_MailBrowse);
 	return false;
     }
     if(cw->sqlMode) {
-	setError("cannot send mail while in database mode");
+	setError(MSG_MailDB);
 	return false;
     }
     if(cw->dirMode) {
-	setError("cannot send mail from directory mode");
+	setError(MSG_MailDir);
 	return false;
     }
     if(cw->binMode) {
-	setError("cannot mail binary data, should this be an attachment?");
+	setError(MSG_MailBinary2);
 	return false;
     }
     if(!cw->dol) {
-	setError("cannot mail an empty file");
+	setError(MSG_MailEmpty);
 	return false;
     }
 
@@ -998,26 +1023,38 @@ sendMailCurrent(int sm_account, bool dosig)
 	char *line = (char *)fetchLine(ln, -1);
 	if(memEqualCI(line, "to:", 3) ||
 	   memEqualCI(line, "mailto:", 7) ||
+	   memEqualCI(line, "cc:", 3) ||
+	   memEqualCI(line, "bcc:", 4) ||
 	   memEqualCI(line, "reply to:", 9) ||
 	   memEqualCI(line, "reply to ", 9)) {
+	    char cc = 0;
+	    if(toupper(line[0]) == 'C')
+		cc = '^';
+	    if(toupper(line[0]) == 'B')
+		cc = '?';
 	    if(toupper(line[0]) == 'R')
 		line += 9;
-	    else if(toupper(line[0]) == 'M')
-		line += 7;
 	    else
-		line += 3;
+		line = strchr(line, ':') + 1;
 	    while(*line == ' ' || *line == '\t')
 		++line;
 	    if(*line == '\n') {
-		setError("no recipient at line %d", ln);
+		setError(MSG_RecipNone2, ln);
 		goto done;
 	    }
 	    if(nrec == MAXRECAT) {
-		setError("too many recipients, limit %d", MAXRECAT);
+		setError(MSG_RecipMany, MAXRECAT);
 		goto done;
 	    }
 	    ++nrec;
 	    for(t = line; *t != '\n'; ++t) ;
+	    if(cc) {
+		if(!lr) {
+		    setError(MSG_MailFirstCC);
+		    goto done;
+		}
+		stringAndChar(&recmem, &lr, cc);
+	    }
 	    stringAndBytes(&recmem, &lr, line, t + 1 - line);
 	    continue;
 	}
@@ -1029,11 +1066,11 @@ sendMailCurrent(int sm_account, bool dosig)
 	    while(*line == ' ' || *line == '\t')
 		++line;
 	    if(*line == '\n') {
-		setError("no attachment at line %d", ln);
+		setError(MSG_AttLineX, ln);
 		goto done;
 	    }
 	    if(nat == MAXRECAT) {
-		setError("too many recipients, limit %d", MAXRECAT);
+		setError(MSG_RecipMany, MAXRECAT);
 		goto done;
 	    }
 	    ++nat;
@@ -1048,7 +1085,7 @@ sendMailCurrent(int sm_account, bool dosig)
 	    if(!isdigitByte(*line) ||
 	       (account = strtol(line, &line, 10)) == 0 ||
 	       account > maxAccount || *line != '\n') {
-		setError("invalid account number at line %d", ln);
+		setError(MSG_MailAccountBadLineX, ln);
 		goto done;
 	    }
 	    continue;
@@ -1057,7 +1094,7 @@ sendMailCurrent(int sm_account, bool dosig)
 	    while(*line == ' ' || *line == '\t')
 		++line;
 	    if(*line == '\n') {
-		setError("empty subject");
+		setError(MSG_SubjectEmpty2);
 		goto done;
 	    }
 	    subj = true;
@@ -1068,16 +1105,12 @@ sendMailCurrent(int sm_account, bool dosig)
     if(sm_account)
 	account = sm_account;
     if(!subj) {
-	setError(ln > cw->dol ?
-	   "there is no subject line" :
-	   "line %d should begin with to: attach: alt: account: or subject:",
-	   ln);
+	setError(((ln > cw->dol) + MSG_MailFirstLine), ln);
 	goto done;
     }
 
     if(nrec == 0) {
-	setError
-	   ("no recipients specified, place to: emailaddress at the top of youre file");
+	setError(MSG_RecipNone3);
 	goto done;
     }
 
@@ -1101,8 +1134,8 @@ sendMailCurrent(int sm_account, bool dosig)
     nzFree(recmem);
     nzFree(atmem);
     if(!rc && intFlag)
-	setError(opint);
+	setError(MSG_Interrupted);
     if(rc)
-	puts("ok");
+	i_puts(MSG_OK);
     return rc;
 }				/* sendMailCurrent */
