@@ -39,7 +39,7 @@ allocMem(unsigned n)
     if(!n)
 	return EMPTYSTRING;
     if(!(s = malloc(n)))
-	errorPrint("@error allocating %u bytes", n);
+	i_printfExit(MSG_MemAllocError, n);
     return s;
 }				/* allocMem */
 
@@ -50,7 +50,7 @@ allocZeroMem(unsigned n)
     if(!n)
 	return EMPTYSTRING;
     if(!(s = calloc(n, 1)))
-	errorPrint("@error callocating %u bytes", n);
+	i_printfExit(MSG_MemCallocError, n);
     return s;
 }				/* allocZeroMem */
 
@@ -59,13 +59,13 @@ reallocMem(void *p, unsigned n)
 {
     void *s;
     if(!n)
-	errorPrint("@reallocMem(p,0)");
+	i_printfExit(MSG_ReallocP);
     if(!p)
-	errorPrint("@reallocMem(0,%d)", n);
+	i_printfExit(MSG_Realloc0, n);
     if(p == EMPTYSTRING)
 	return allocMem(n);
     if(!(s = realloc(p, n)))
-	errorPrint("@error re-allocating %u bytes", n);
+	i_printfExit(MSG_ErrorRealloc, n);
     return s;
 }				/* reallocMem */
 
@@ -438,7 +438,7 @@ stringInList(const char *const *list, const char *s)
 {
     int i = 0;
     if(!list)
-	errorPrint("@stringInList(null,...)");
+	i_printfExit(MSG_NullStrList);
     if(s)
 	while(*list) {
 	    if(stringEqual(s, *list))
@@ -454,7 +454,7 @@ stringInListCI(const char *const *list, const char *s)
 {
     int i = 0;
     if(!list)
-	errorPrint("@stringInListCI(null,...)");
+	i_printfExit(MSG_NullStrListCI);
     if(s)
 	while(*list) {
 	    if(stringEqualCI(s, *list))
@@ -470,7 +470,7 @@ charInList(const char *list, char c)
 {
     char *s;
     if(!list)
-	errorPrint("@charInList(null,...)");
+	i_printfExit(MSG_NullCharInList);
     s = strchr(list, c);
     if(!s)
 	return -1;
@@ -555,59 +555,28 @@ isquote(char c)
     return c == '"' || c == '\'';
 }				/* isquote */
 
-/* Gather at most 5 parameters from a vararg list
- * and place them in local variables.
- * Only take as many as indicated by the percents in a sprintf string.
- * If we blindly take more, we risk core dumps, at least on the Sun. */
-
-void
-varargLocals(va_list p, const char *msg, long *locals)
-{
-    const char *s = msg;
-    int cnt;
-
-    for(cnt = 0; cnt < 5; msg = s) {
-	s = strchr(msg, '%');
-	if(!s)
-	    break;
-	++s;
-	if(*s == '%') {
-	    ++s;
-	    continue;
-	}
-	if(strchr("-.0123456789lhusdfxco", *s)) {
-	    long n = va_arg(p, long);
-	    locals[cnt++] = n;
-	}
-    }				/* while finding percents in msg */
-
-/* a little protection, in case they pass too few arguments */
-    while(cnt < 5)
-	locals[cnt++] = (long)"argmissing";
-}				/* varargLocals */
-
 /* print an error message */
 void
 errorPrint(const char *msg, ...)
 {
     char bailflag = 0;
     va_list p;
-    long a[5];
+
     va_start(p, msg);
-    varargLocals(p, msg, a);
-    va_end(p);
 
     if(*msg == '@') {
-	++msg;
 	bailflag = 1;
+	++msg;
+/* I should internationalize this, but it's never suppose to happen! */
 	fprintf(stderr, "disaster, ");
     } else if(isdigitByte(*msg)) {
-	++msg;
 	bailflag = *msg - '0';
+	++msg;
     }
 
-    fprintf(stderr, msg, a[0], a[1], a[2], a[3], a[4]);
+    vfprintf(stderr, msg, p);
     fprintf(stderr, "\n");
+    va_end(p);
 
     if(bailflag)
 	exit(bailflag);
@@ -617,111 +586,21 @@ void
 debugPrint(int lev, const char *msg, ...)
 {
     va_list p;
-    long a[5];
     if(lev > debugLevel)
 	return;
     va_start(p, msg);
-    varargLocals(p, msg, a);
+    vprintf(msg, p);
     va_end(p);
-    printf(msg, a[0], a[1], a[2], a[3], a[4]);
-    printf("\n");
+    nl();
     if(lev == 0 && !memcmp(msg, "warning", 7))
 	eeCheck();
 }				/* debugPrint */
 
-char errorMsg[4000];
-/* Show the error message, not just the question mark, after these commands. */
-static const char showerror_cmd[] = "AbefMqrw^";
-
-/* Set the error message.  Type h to see the message. */
 void
-setError(const char *msg, ...)
+nl(void)
 {
-    va_list p;
-    long a[5];
-
-    if(!msg) {
-	errorMsg[0] = 0;
-	return;
-    }
-
-    va_start(p, msg);
-    varargLocals(p, msg, a);
-    va_end(p);
-/* Yeah I know, there's a va_list sprintf call; this is old code. */
-    sprintf(errorMsg, msg, a[0], a[1], a[2], a[3], a[4]);
-
-/* sanity check */
-    if(strlen(errorMsg) >= sizeof (errorMsg)) {
-	printf("disaster, error message, length %d is too long\n",
-	   strlen(errorMsg));
-	puts(errorMsg);
-	exit(1);
-    }
-}				/* setError */
-
-void
-showError(void)
-{
-    printf("%s\n", errorMsg[0] ? errorMsg : "no errors");
-}				/* showError */
-
-void
-showErrorConditional(char cmd)
-{
-    if(helpMessagesOn || strchr(showerror_cmd, cmd))
-	showError();
-    else
-	printf("?\n");
-}				/* showErrorConditional */
-
-void
-showErrorAbort(void)
-{
-    errorPrint("1%s", errorMsg);
-}				/* showErrorAbort */
-
-void
-browseError(const char *msg, ...)
-{
-    va_list p;
-    long a[5];
-    if(ismc)
-	return;
-    if(browseLocal != 1)
-	return;
-    va_start(p, msg);
-    varargLocals(p, msg, a);
-    va_end(p);
-    if(browseLine) {
-	printf("line %d: ", browseLine);
-	cw->labels[4] = browseLine;
-    } else
-	printf("browse error: ");
-    printf(msg, a[0], a[1], a[2], a[3], a[4]);
-    printf("\n");
-    browseLocal = 2;
-}				/* browseError */
-
-/* Javascript errors, we need to see these no matter what. */
-void
-runningError(const char *msg, ...)
-{
-    va_list p;
-    long a[5];
-    if(ismc)
-	return;
-    va_start(p, msg);
-    varargLocals(p, msg, a);
-    va_end(p);
-    if(browseLine) {
-	printf("line %d: ", browseLine);
-	cw->labels[4] = browseLine;
-    }
-    printf(msg, a[0], a[1], a[2], a[3], a[4]);
-    printf("\n");
-    browseLocal = 2;
-}				/* runningError */
+    puts("");
+}				/* nl */
 
 /* Turn perl string into C string, and complain about nulls. */
 int
@@ -743,7 +622,7 @@ pstLength(pst s)
 {
     pst t;
     if(!s)
-	errorPrint("@null pointer in pstLength");
+	i_printfExit(MSG_NullPtr);
     t = s;
     while(*t != '\n')
 	++t;
@@ -777,12 +656,12 @@ fileIntoMemory(const char *filename, char **data, int *len)
     char *buf;
     char ftype = fileTypeByName(filename, false);
     if(ftype && ftype != 'f') {
-	setError("%s is not a regular file", filename);
+	setError(MSG_RegularFile, filename);
 	return false;
     }
     fh = open(filename, O_RDONLY | O_BINARY);
     if(fh < 0) {
-	setError("cannot open %s", filename);
+	setError(MSG_NoOpen, filename);
 	return false;
     }
     length = fileSizeByName(filename);
@@ -791,7 +670,7 @@ fileIntoMemory(const char *filename, char **data, int *len)
 	return false;
     }				/* should never hapen */
     if(length > maxFileSize) {
-	setError("file is too large, limit 40MB");
+	setError(MSG_LargeFile);
 	close(fh);
 	return false;
     }
@@ -801,7 +680,7 @@ fileIntoMemory(const char *filename, char **data, int *len)
 	n = read(fh, buf, length);
     close(fh);			/* don't need that any more */
     if(n < length) {
-	setError("cannot read the contents of %s", filename);
+	setError(MSG_NoRead2, filename);
 	free(buf);
 	return false;
     }
@@ -873,7 +752,7 @@ fileTypeByName(const char *name, bool showlink)
     char c;
     int mode;
     if(lstat(name, &buf)) {
-	setError("cannot access %s", name);
+	setError(MSG_NoAccess, name);
 	return 0;
     }
     mode = buf.st_mode & S_IFMT;
@@ -908,7 +787,7 @@ fileSizeByName(const char *name)
 {
     struct stat buf;
     if(stat(name, &buf)) {
-	setError("cannot access %s", name);
+	setError(MSG_NoAccess, name);
 	return -1;
     }
     return buf.st_size;
@@ -919,7 +798,7 @@ fileTimeByName(const char *name)
 {
     struct stat buf;
     if(stat(name, &buf)) {
-	setError("cannot access %s", name);
+	setError(MSG_NoAccess, name);
 	return -1;
     }
     return buf.st_mtime;
@@ -934,7 +813,7 @@ ttySaveSettings(void)
     isInteractive = isatty(0);
     if(isInteractive) {
 	if(ioctl(0, TTY_GET_COMMAND, &savettybuf))
-	    errorPrint("@canot use ioctl() to manage the tty");
+	    i_printfExit(MSG_IoctlError);
     }
 }				/* ttySaveSettings */
 
@@ -1058,8 +937,7 @@ nextScanFile(const char *base)
 	    base = ".";
 	df = opendir(base);
 	if(!df) {
-	    puts
-	       ("warning, the directory is inaccessible - the listing will be empty");
+	    i_puts(MSG_NoDirNoList);
 	    return 0;
 	}
     }
@@ -1202,7 +1080,7 @@ envFile(const char *line, const char **expanded)
 	    *t = 0;
 	    value = getenv(dollar + 1);
 	    if(!value) {
-		setError("environement variable %s not set", dollar + 1);
+		setError(MSG_NoEnvVar, dollar + 1);
 		return false;
 	    }
 	    if(dollar + strlen(value) >= line1 + sizeof (line1) - 1)
@@ -1236,7 +1114,7 @@ envFile(const char *line, const char **expanded)
 	return true;
     }
     if(cut && dollar < cut) {
-	setError("cannot expand * ? or [] prior to the last /");
+	setError(MSG_EarlyExpand);
 	return false;
     }
 
@@ -1244,7 +1122,7 @@ envFile(const char *line, const char **expanded)
     if(cut) {
 	*cut = 0;
 	if(cut > line1 && fileTypeByName(line1, false) != 'd') {
-	    setError("%s is not an accessible directory", line1);
+	    setError(MSG_NoAccessDir, line1);
 	    return false;
 	}
     }
@@ -1255,12 +1133,11 @@ envFile(const char *line, const char **expanded)
     cc = badBrackets = false;
     while(c = *s) {
 	if(t >= re + sizeof (re) - 3) {
-	    setError("shell pattern is too long");
+	    setError(MSG_ShellPatternLong);
 	    return false;
 	}
 	if(c == '\\') {
-	    setError
-	       ("sorry, I don't know how to expand filenames with \\ in them");
+	    setError(MSG_ExpandBackslash);
 	    return false;
 	}
 /* things we need to escape */
@@ -1286,14 +1163,14 @@ envFile(const char *line, const char **expanded)
     *t++ = '$';
     *t = 0;
     if(badBrackets | cc) {
-	setError("improperly formed [] pattern");
+	setError(MSG_ShellSyntax);
 	return false;
     }
 
     debugPrint(7, "shell regexp %s", re);
     re_cc = pcre_compile(re, 0, &re_error, &re_offset, 0);
     if(!re_cc) {
-	setError("error compiling the shell pattern, %s", re_error);
+	setError(MSG_ShellCompile, re_error);
 	return false;
     }
 
@@ -1312,7 +1189,7 @@ envFile(const char *line, const char **expanded)
 	re_count = pcre_exec(re_cc, 0, file, strlen(file), 0, 0, re_vector, 3);
 	if(re_count < -1) {
 	    pcre_free(re_cc);
-	    setError("unexpected error while evaluating the shell pattern");
+	    setError(MSG_ShellExpand);
 	    return false;
 	}
 	if(re_count < 0)
@@ -1329,8 +1206,7 @@ envFile(const char *line, const char **expanded)
     }
     pcre_free(re_cc);
     if(filecount != 1) {
-	setError(filecount ? "shell pattern matches more than one file" :
-	   "shell pattern does not match any files");
+	setError((filecount > 0) + MSG_ShellNoMatch);
 	return false;
     }
     if(cc)
@@ -1340,7 +1216,7 @@ envFile(const char *line, const char **expanded)
     return true;
 
   longvar:
-    setError("line becomes too long when shell variables are expanded");
+    setError(MSG_ShellLineLong);
     return false;
 }				/* envFile */
 
@@ -1377,11 +1253,11 @@ efopen(const char *name, const char *mode)
 	return f;
 
     if(*mode == 'r')
-	errorPrint("2cannot open %s", name);
+	i_printfExit(MSG_OpenFail, name);
     else if(*mode == 'w' || *mode == 'a')
-	errorPrint("2cannot create or write to %s", name);
+	i_printfExit(MSG_CreateFail, name);
     else
-	errorPrint("@calling fopen() with invalid mode %s", mode);
+	i_printfExit(MSG_InvalidFopen, mode);
     return 0;
 }				/* efopen */
 
@@ -1390,14 +1266,10 @@ appendFile(const char *fname, const char *message, ...)
 {
     FILE *f;
     va_list p;
-    long a[5];
-
-    va_start(p, message);
-    varargLocals(p, message, a);
-    va_end(p);
-
     f = efopen(fname, "a");
-    fprintf(f, message, a[0], a[1], a[2], a[3], a[4]);
+    va_start(p, message);
+    vfprintf(f, message, p);
+    va_end(p);
     fprintf(f, "\n");
     fclose(f);
 }				/* appendFile */
