@@ -184,7 +184,7 @@ parseURL(const char *url, const char **proto, int *prlen, const char **user, int
 	if(p[1] != '/' || p[2] != '/') {
 	    if(protocols[a].need_slashes) {
 		if(p[1] != '/') {
-		    setError("%s:// expected", protocols[a].prot);
+		    setError(MSG_ProtExpected, protocols[a].prot);
 		    return -1;
 		}
 /* We got one out of two slashes, I'm going to call it good */
@@ -202,7 +202,7 @@ parseURL(const char *url, const char **proto, int *prlen, const char **user, int
 		char qprot[12];
 		memcpy(qprot, url, p - url);
 		qprot[p - url] = 0;
-		setError("unrecognized protocol %s", qprot);
+		setError(MSG_BadProt, qprot);
 		return -1;
 	    }
 	}
@@ -259,7 +259,7 @@ parseURL(const char *url, const char **proto, int *prlen, const char **user, int
 	const char *cc, *pp = q + strcspn(q, "/?#\1");
 	n = strtol(q + 1, (char **)&cc, 10);
 	if(cc != pp || !isdigitByte(q[1])) {
-	    setError("invalid :port specifier at the end of the domain");
+	    setError(MSG_BadPort);
 	    return -1;
 	}
 	if(port)
@@ -331,7 +331,7 @@ getHostURL(const char *url)
     if(!s)
 	return EMPTYSTRING;
     if(l >= sizeof (hostbuf)) {
-	setError("domain name too long");
+	setError(MSG_DomainLong);
 	return 0;
     }
     memcpy(hostbuf, s, l);
@@ -368,7 +368,7 @@ getHostPassURL(const char *url)
     if(u)
 	z = u, hl += h - u, t += h - u;
     if(hl >= sizeof (hostbuf)) {
-	setError("domain name too long");
+	setError(MSG_DomainLong);
 	return 0;
     }
     memcpy(hostbuf, z, hl);
@@ -393,7 +393,7 @@ getUserURL(const char *url)
     if(!s)
 	return EMPTYSTRING;
     if(l >= sizeof (buf)) {
-	setError("user name too long");
+	setError(MSG_UserNameLong2);
 	return 0;
     }
     memcpy(buf, s, l);
@@ -415,7 +415,7 @@ getPassURL(const char *url)
     if(!s)
 	return EMPTYSTRING;
     if(l >= sizeof (buf)) {
-	setError("password too long");
+	setError(MSG_PasswordLong2);
 	return 0;
     }
     memcpy(buf, s, l);
@@ -445,7 +445,7 @@ getDirURL(const char *url, const char **start_p, const char **end_p)
 	goto slash;
     if(!strchr("#?\1", *dir)) {
 	if(*--dir != '/')
-	    errorPrint("1misplaced / in getDirURL(%s)", url);
+	    i_printfExit(MSG_BadDirSlash, url);
     }
     end = strpbrk(dir, "#?\1");
     if(!end)
@@ -518,8 +518,7 @@ fetchHistory(const char *prev, const char *next)
 	    }
     }
     if(n_fetch >= MAXFETCH) {
-	setError
-	   ("too many fetches from the internet, you may want to disable `redirect html'");
+	setError(MSG_WebFetchMany);
 	return -1;
     }
 /* Have we seen this one before? */
@@ -539,7 +538,7 @@ fetchHistory(const char *prev, const char *next)
 	from = histFrom[from];
     if(from < 0)
 	return false;
-    setError("web page indirectly fetches itself, an infinite loop");
+    setError(MSG_WebFetchSelf);
     return -1;
 }				/* FetchHistory */
 
@@ -566,7 +565,7 @@ squashDirectories(char *url)
 	return;
     --dd;
     if(*dd != '/')
-	errorPrint("@misplaced / in %s", url);
+	i_printfExit(MSG_BadSlash, url);
     end = dd + strcspn(dd, "?#\1");
     while(true) {
 	s = strstr(dd, "/./");
@@ -852,7 +851,7 @@ decodePostData(const char *data, const char *name, int seqno)
     char c;
 
     if(!seqno && !name)
-	errorPrint("@decodePostData(0,0)");
+	i_printfExit(MSG_DecodePost);
 
     for(s = data; *s; s = (*t ? t + 1 : t)) {
 	n = 0;
